@@ -1,35 +1,27 @@
 import ast
 import datetime
-import gzip
-import json
 import logging
 import os.path
+import subprocess
 import sys
-import tempfile
 import textwrap
 import tkinter as tk
-import urllib.request
-import webbrowser
 from collections import namedtuple
 from tkinter import messagebox, ttk
-from typing import (
-    Dict,  # pylint disable=unused-import
-    Iterable,
-    List,  # pylint disable=unused-import
-    Optional,  # pylint disable=unused-import
-    Tuple,  # pylint disable=unused-import
-    Type,  # pylint disable=unused-import
-    Union,  # pylint disable=unused-import
-)
-
+from typing import Dict  # pylint disable=unused-import
+from typing import List  # pylint disable=unused-import
+from typing import Optional  # pylint disable=unused-import
+from typing import Tuple  # pylint disable=unused-import
+from typing import Type  # pylint disable=unused-import
+from typing import Union  # pylint disable=unused-import
+from typing import Iterable
 
 import thonny
-from thonny import get_workbench, rst_utils, tktextext, ui_utils, get_runner
+from thonny import get_runner, get_workbench, rst_utils, tktextext, ui_utils
 from thonny.common import ToplevelResponse, read_source
+from thonny.languages import tr
 from thonny.misc_utils import levenshtein_damerau_distance, running_on_mac_os
-from thonny.ui_utils import scrollbar_style, CommonDialog
-from thonny.running import CPythonProxy
-import subprocess
+from thonny.ui_utils import CommonDialog, scrollbar_style
 
 Suggestion = namedtuple("Suggestion", ["symbol", "title", "body", "relevance"])
 
@@ -111,6 +103,8 @@ class AssistantView(tktextext.TextFrame):
 
         self._clear()
 
+        from thonny.plugins.cpython import CPythonProxy
+
         if not isinstance(get_runner().get_backend_proxy(), CPythonProxy):
             # TODO: add some support for MicroPython as well
             return
@@ -582,8 +576,8 @@ class FeedbackDialog(CommonDialog):
         )
         tree_frame.grid(row=3, column=0, columnspan=3, sticky="nsew", padx=padx)
         self.tree = tree_frame.tree
-        self.tree.column("helpful", width=30, anchor=tk.CENTER, stretch=False)
-        self.tree.column("confusing", width=30, anchor=tk.CENTER, stretch=False)
+        self.tree.column("helpful", width=35, anchor=tk.CENTER, stretch=False)
+        self.tree.column("confusing", width=35, anchor=tk.CENTER, stretch=False)
         self.tree.column("title", width=350, anchor=tk.W, stretch=True)
 
         self.tree.heading("helpful", text="H", anchor=tk.CENTER)
@@ -722,8 +716,10 @@ class FeedbackDialog(CommonDialog):
         self.tree.item(item_id, values=tuple(values))
 
     def _preview_submission_data(self, event=None):
+        import tempfile
+
         temp_path = os.path.join(
-            tempfile.mkdtemp(),
+            tempfile.mkdtemp(dir=get_workbench().get_temp_dir()),
             "ThonnyAssistantFeedback_"
             + datetime.datetime.now().isoformat().replace(":", ".")[:19]
             + ".txt",
@@ -735,9 +731,13 @@ class FeedbackDialog(CommonDialog):
         if running_on_mac_os():
             subprocess.Popen(["open", "-e", temp_path])
         else:
+            import webbrowser
+
             webbrowser.open(temp_path)
 
     def _collect_submission_data(self):
+        import json
+
         tree_data = []
 
         for iid in self.tree.get_children():
@@ -785,6 +785,9 @@ class FeedbackDialog(CommonDialog):
         return json.dumps(submission, indent=2)
 
     def _submit_data(self):
+        import gzip
+        import urllib.request
+
         json_data = self._collect_submission_data()
         compressed_data = gzip.compress(json_data.encode("ascii"))
 
@@ -815,7 +818,7 @@ class FeedbackDialog(CommonDialog):
                 "Problem",
                 "Something went wrong:\n%s\n\nIf you don't mind, then try again later!"
                 % result[:1000],
-                master=self.master,
+                master=self,
             )
 
     def _select_unsent_snapshots(self, all_snapshots):
@@ -962,4 +965,4 @@ def init():
     get_workbench().set_default("assistance.open_assistant_on_errors", True)
     get_workbench().set_default("assistance.open_assistant_on_warnings", False)
     get_workbench().set_default("assistance.disabled_checks", [])
-    get_workbench().add_view(AssistantView, _("Assistant"), "se", visible_by_default=False)
+    get_workbench().add_view(AssistantView, tr("Assistant"), "se", visible_by_default=False)
