@@ -3,19 +3,30 @@
 """
 Classes used both by front-end and back-end
 """
+import logging
 import os.path
 import platform
 import site
-import sys
-import tokenize
-from collections import namedtuple
-from typing import List, Optional  # @UnusedImport
 import subprocess
-import logging
+import sys
 import traceback
+from collections import namedtuple
 from threading import Thread
+from typing import List, Optional, Dict, Iterable, Tuple  # @UnusedImport
+
+logger = logging.getLogger(__name__)
 
 MESSAGE_MARKER = "\x02"
+OBJECT_LINK_START = "[object_link_for_thonny]"
+OBJECT_LINK_END = "[/object_link_for_thonny]"
+
+IGNORED_FILES_AND_DIRS = [
+    "System Volume Information",
+    "._.Trashes",
+    ".Trashes",
+    "__MACOSX",
+    ".DS_Store",
+]
 
 ValueInfo = namedtuple("ValueInfo", ["id", "repr"])
 FrameInfo = namedtuple(
@@ -145,11 +156,8 @@ class CommandToBackend(Record):
         self.name = name
 
 
-class InterruptCommand(CommandToBackend):
-    def __init__(self, **kw) -> None:
-        if "name" in kw:
-            del kw["name"]
-        super().__init__("interrupt", **kw)
+class ImmediateCommand(CommandToBackend):
+    pass
 
 
 class EOFCommand(CommandToBackend):
@@ -210,7 +218,7 @@ class InlineResponse(MessageFromBackend):
 def serialize_message(msg: Record) -> str:
     # I want to transfer only ASCII chars because encodings are not reliable
     # (eg. can't find a way to specify PYTHONIOENCODING for cx_freeze'd program)
-    return MESSAGE_MARKER + repr(msg).encode("UTF-7").decode("ASCII")
+    return MESSAGE_MARKER + ascii(msg)
 
 
 def parse_message(msg_string: str) -> Record:
@@ -218,11 +226,14 @@ def parse_message(msg_string: str) -> Record:
     # pylint: disable=unused-variable
     nan = float("nan")  # @UnusedVariable
     assert msg_string[0] == MESSAGE_MARKER
-    return eval(msg_string[1:].encode("ASCII").decode("UTF-7"))
+    return eval(msg_string[1:])
 
 
 def normpath_with_actual_case(name: str) -> str:
     """In Windows return the path with the case it is stored in the filesystem"""
+    if not os.path.exists(name):
+        return os.path.normpath(name)
+
     assert os.path.isabs(name) or os.path.ismount(name), "Not abs nor mount: " + name
     assert os.path.exists(name), "Not exists: " + name
 
@@ -271,6 +282,8 @@ def path_startswith(child_name: str, dir_name: str) -> bool:
 
 
 def read_source(filename):
+    import tokenize
+
     with tokenize.open(filename) as fp:
         return fp.read()
 
@@ -375,24 +388,18 @@ def is_hidden_or_system_file(path: str) -> bool:
         return False
 
 
-def get_dirs_child_data(paths):
-    """Used for populating local file browser's tree view.
-    dir_paths contains full paths of the open directories.
-    Returns information required for refreshing this view"""
-    res = {}
-    for path in paths:
-        # assuming the path already has actual case
-        res[path] = get_single_dir_child_data(path)
-
-    return res
+def get_dirs_children_info(
+    paths: List[str], include_hidden: bool = False
+) -> Dict[str, Optional[Dict[str, Dict]]]:
+    return {path: get_single_dir_child_data(path, include_hidden) for path in paths}
 
 
-def get_single_dir_child_data(path):
+def get_single_dir_child_data(path: str, include_hidden: bool = False) -> Optional[Dict[str, Dict]]:
     if path == "":
         if platform.system() == "Windows":
             return {**get_windows_volumes_info(), **get_windows_network_locations()}
         else:
-            return get_single_dir_child_data("/")
+            return get_single_dir_child_data("/", include_hidden)
 
     elif os.path.isdir(path) or os.path.ismount(path):
         result = {}
@@ -404,15 +411,22 @@ def get_single_dir_child_data(path):
                     # must be broken link
                     continue
                 full_child_path = normpath_with_actual_case(full_child_path)
-                if not is_hidden_or_system_file(full_child_path):
-                    st = os.stat(full_child_path, dir_fd=None, follow_symlinks=True)
+                hidden = is_hidden_or_system_file(full_child_path)
+                if not hidden or include_hidden:
                     name = os.path.basename(full_child_path)
+                    st = os.stat(full_child_path, dir_fd=None, follow_symlinks=True)
                     result[name] = {
                         "size": None if os.path.isdir(full_child_path) else st.st_size,
-                        "time": max(st.st_mtime, st.st_ctime),
+                        "modified": st.st_mtime,
+                        "hidden": hidden,
                     }
         except PermissionError:
-            result["<not accessible>"] = {"kind": "error", "size": -1, "time": None}
+            result["<not accessible>"] = {
+                "kind": "error",
+                "size": -1,
+                "modified": None,
+                "hidden": None,
+            }
 
         return result
     else:
@@ -423,7 +437,6 @@ def get_windows_volumes_info():
     # http://stackoverflow.com/a/2288225/261181
     # http://msdn.microsoft.com/en-us/library/windows/desktop/aa364939%28v=vs.85%29.aspx
     import string
-
     from ctypes import windll
 
     all_drive_types = [
@@ -472,12 +485,10 @@ def get_windows_volumes_info():
                     result[path] = {
                         "label": label,
                         "size": None,
-                        "time": max(st.st_mtime, st.st_ctime),
+                        "modified": max(st.st_mtime, st.st_ctime),
                     }
-
-                except PermissionError:
-                    traceback.print_exc()
-                    # probably an empty cardreader slot
+                except OSError as e:
+                    logger.warning("Could not get information for %s", path, exc_info=e)
 
         bitmask >>= 1
 
@@ -531,12 +542,10 @@ def get_windows_network_locations():
                 result[target] = {
                     "label": entry.name + " (" + target + ")",
                     "size": None,
-                    "time": None,
+                    "modified": None,
                 }
-            except:
-                logging.getLogger("thonny").error(
-                    "Can't get target from %s", lnk_path, exc_info=True
-                )
+            except Exception:
+                logger.error("Can't get target from %s", lnk_path, exc_info=True)
 
     return result
 
@@ -552,6 +561,7 @@ def get_windows_lnk_target(lnk_file_path):
 
 
 def execute_system_command(cmd, cwd=None, disconnect_stdin=False):
+    logger.debug("execute_system_command, cmd=%r, cwd=%s", cmd, cwd)
     env = dict(os.environ).copy()
     encoding = "utf-8"
     env["PYTHONIOENCODING"] = encoding
@@ -559,9 +569,6 @@ def execute_system_command(cmd, cwd=None, disconnect_stdin=False):
     # in PATH
     update_system_path(env, get_augmented_system_path(get_exe_dirs()))
     popen_kw = dict(
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
-        shell=True,
         env=env,
         universal_newlines=True,
         bufsize=0,
@@ -577,28 +584,63 @@ def execute_system_command(cmd, cwd=None, disconnect_stdin=False):
         popen_kw["errors"] = "replace"
         popen_kw["encoding"] = encoding
 
-    assert cmd.cmd_line.startswith("!")
-    cmd_line = cmd.cmd_line[1:]
+    if isinstance(cmd.cmd_line, str) and cmd.cmd_line.startswith("!"):
+        cmd_line = cmd.cmd_line[1:]
+        popen_kw["shell"] = True
+    else:
+        assert isinstance(cmd.cmd_line, list)
+        cmd_line = cmd.cmd_line
+    logger.debug("Popen(%r, ...)", cmd_line)
     proc = subprocess.Popen(cmd_line, **popen_kw)
+    proc.communicate()
+    return proc.wait()
 
-    def copy_stream(source, target):
-        while True:
-            c = source.readline()
-            if c == "":
-                break
-            else:
-                target.write(c)
-                target.flush()
 
-    copy_out = Thread(target=lambda: copy_stream(proc.stdout, sys.stdout), daemon=True)
-    copy_err = Thread(target=lambda: copy_stream(proc.stderr, sys.stderr), daemon=True)
+def universal_dirname(path: str) -> str:
+    if "/" in path:
+        sep = "/"
+    elif "\\" in path:
+        sep = "\\"
+    else:
+        # micro:bit
+        return ""
 
-    copy_out.start()
-    copy_err.start()
-    try:
-        proc.wait()
-    except KeyboardInterrupt as e:
-        print(str(e), file=sys.stderr)
+    path = path.rstrip(sep)
+    result = path[: path.rindex(sep)]
+    if not result:
+        return sep
+    else:
+        return result
 
-    copy_out.join()
-    copy_err.join()
+
+def universal_relpath(path: str, context: str) -> str:
+    """Tries to give relative path"""
+    if "/" in path:
+        import pathlib
+
+        p = pathlib.PurePosixPath(path)
+        try:
+            return str(p.relative_to(context))
+        except ValueError:
+            return path
+    else:
+        return os.path.relpath(path, context)
+
+
+def get_python_version_string(version_info: Optional[Tuple] = None, maxsize=None):
+    result = ".".join(map(str, sys.version_info[:3]))
+    if sys.version_info[3] != "final":
+        result += "-" + sys.version_info[3]
+
+    if maxsize is not None:
+        result += " (" + ("64" if sys.maxsize > 2 ** 32 else "32") + " bit)"
+
+    return result
+
+
+class ConnectionFailedException(Exception):
+    pass
+
+
+class ConnectionClosedException(Exception):
+    pass
