@@ -1,24 +1,29 @@
 import tkinter as tk
+import traceback
 from tkinter import ttk
+from typing import Optional
 
 from thonny import get_workbench, ui_utils
-import traceback
-from thonny.ui_utils import ems_to_pixels, CommonDialog
+from thonny.languages import tr
+from thonny.ui_utils import CommonDialog, ems_to_pixels
 
 
 class ConfigurationDialog(CommonDialog):
     last_shown_tab_index = 0
 
-    def __init__(self, master, page_records):
+    def __init__(self, master, page_records_with_order):
         super().__init__(master)
         width = ems_to_pixels(53)
         height = ems_to_pixels(43)
         self.geometry("%dx%d" % (width, height))
-        self.title(_("Thonny options"))
+        self.title(tr("Thonny options"))
 
         self.columnconfigure(0, weight=1)
         self.rowconfigure(0, weight=1)
 
+        self.backend_restart_required = False
+        self.gui_restart_required = False
+
         main_frame = ttk.Frame(self)  # otherwise there is wrong color background with clam
         main_frame.grid(row=0, column=0, sticky=tk.NSEW)
         main_frame.columnconfigure(0, weight=1)
@@ -27,19 +32,22 @@ class ConfigurationDialog(CommonDialog):
         self._notebook = ttk.Notebook(main_frame)
         self._notebook.grid(row=0, column=0, columnspan=3, sticky=tk.NSEW, padx=10, pady=10)
 
-        self._ok_button = ttk.Button(main_frame, text=_("OK"), command=self._ok, default="active")
-        self._cancel_button = ttk.Button(main_frame, text=_("Cancel"), command=self._cancel)
+        self._ok_button = ttk.Button(main_frame, text=tr("OK"), command=self._ok, default="active")
+        self._cancel_button = ttk.Button(main_frame, text=tr("Cancel"), command=self._cancel)
         self._ok_button.grid(row=1, column=1, padx=(0, 11), pady=(0, 10))
         self._cancel_button.grid(row=1, column=2, padx=(0, 11), pady=(0, 10))
 
         self._page_records = []
-        for key, title, page_class, order in sorted(page_records, key=lambda r: (r[3], r[0])):
+        for key, title, page_class, _ in sorted(
+            page_records_with_order, key=lambda r: (r[3], r[0])
+        ):
             try:
                 spacer = ttk.Frame(self)
                 spacer.rowconfigure(0, weight=1)
                 spacer.columnconfigure(0, weight=1)
                 page = page_class(spacer)
                 page.key = key
+                page.dialog = self
                 self._page_records.append((key, title, page))
                 page.grid(sticky=tk.NSEW, pady=(15, 10), padx=15)
                 self._notebook.add(spacer, text=title)
@@ -57,9 +65,9 @@ class ConfigurationDialog(CommonDialog):
                 self._notebook.select(tab)
 
     def _ok(self, event=None):
-        for key, title, page in self._page_records:
+        for _, title, page in self._page_records:
             try:
-                if page.apply() == False:
+                if page.apply() is False:
                     return
             except Exception:
                 get_workbench().report_exception("Error when applying options in " + title)
@@ -67,7 +75,7 @@ class ConfigurationDialog(CommonDialog):
         self.destroy()
 
     def _cancel(self, event=None):
-        for key, title, page in self._page_records:
+        for _, title, page in self._page_records:
             try:
                 page.cancel()
             except Exception:
@@ -82,13 +90,14 @@ class ConfigurationDialog(CommonDialog):
 
 class ConfigurationPage(ttk.Frame):
     """This is an example dummy implementation of a configuration page.
-    
+
     It's not required that configuration pages inherit from this class
     (can be any widget), but the class must have constructor with single parameter
     for getting the master."""
 
     def __init__(self, master):
         ttk.Frame.__init__(self, master)
+        self.dialog = None  # type: Optional[ConfigurationDialog]
 
     def add_checkbox(
         self, flag_name, description, row=None, column=0, padx=0, pady=0, columnspan=1, tooltip=None
@@ -103,9 +112,10 @@ class ConfigurationPage(ttk.Frame):
             ui_utils.create_tooltip(checkbox, tooltip)
 
     def add_combobox(
-        self, option_name, values, row=None, column=0, padx=0, pady=0, columnspan=1, width=None
+        self, variable, values, row=None, column=0, padx=0, pady=0, columnspan=1, width=None
     ):
-        variable = get_workbench().get_variable(option_name)
+        if isinstance(variable, str):
+            variable = get_workbench().get_variable(variable)
         combobox = ttk.Combobox(
             self,
             exportselection=False,
@@ -118,6 +128,7 @@ class ConfigurationPage(ttk.Frame):
         combobox.grid(
             row=row, column=column, sticky=tk.W, pady=pady, padx=padx, columnspan=columnspan
         )
+        return variable
 
     def add_entry(self, option_name, row=None, column=0, pady=0, padx=0, columnspan=1, **kw):
         variable = get_workbench().get_variable(option_name)
