@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
 
 import ast
+import sys
 import tkinter as tk
 import traceback
 
 from thonny import ast_utils, get_workbench, ui_utils
 from thonny.common import TextRange, range_contains_smaller
-import sys
+from thonny.languages import tr
 
 
 class AstView(ui_utils.TreeFrame):
@@ -78,7 +79,7 @@ class AstView(ui_utils.TreeFrame):
         def _format(key, node, parent_id):
 
             if isinstance(node, ast.AST):
-                fields = [(key, val) for key, val in ast.iter_fields(node)]
+                fields = list(ast.iter_fields(node))
 
                 value_label = node.__class__.__name__
 
@@ -164,7 +165,7 @@ def _find_closest_containing_node(tree, text_range):
 def pretty(node, key="/", level=0):
     """Used for exporting ASTView content"""
     if isinstance(node, ast.AST):
-        fields = [(key, val) for key, val in ast.iter_fields(node)]
+        fields = list(ast.iter_fields(node))
         value_label = node.__class__.__name__
         if isinstance(node, ast.Call):
             # Try to make 3.4 AST-s more similar to 3.5
@@ -206,4 +207,4 @@ def pretty(node, key="/", level=0):
 
 
 def load_plugin() -> None:
-    get_workbench().add_view(AstView, _("Program tree"), "s")
+    get_workbench().add_view(AstView, tr("Program tree"), "s")
