@@ -12,13 +12,26 @@ from tkinter import ttk
 from tkinter.messagebox import showinfo
 from typing import List, Union  # @UnusedImport
 
-from thonny import ast_utils, code, get_runner, get_workbench, memory, misc_utils, ui_utils, running
-from thonny.codeview import CodeView, get_syntax_options_for_tag, SyntaxText
+from _tkinter import TclError
+
+from thonny import (
+    ast_utils,
+    editors,
+    get_runner,
+    get_workbench,
+    memory,
+    misc_utils,
+    running,
+    ui_utils,
+)
+from thonny.codeview import CodeView, SyntaxText, get_syntax_options_for_tag
 from thonny.common import DebuggerCommand, InlineCommand
+from thonny.languages import tr
 from thonny.memory import VariablesFrame
-from thonny.misc_utils import shorten_repr, running_on_rpi, running_on_mac_os
+from thonny.misc_utils import running_on_mac_os, running_on_rpi, shorten_repr
 from thonny.tktextext import TextFrame
-from thonny.ui_utils import select_sequence, CommonDialog
+from thonny.ui_utils import CommonDialog, select_sequence
+from thonny.ui_utils import select_sequence, CommonDialog, get_tk_version_info
 from _tkinter import TclError
 
 _current_debugger = None
@@ -39,7 +52,7 @@ class Debugger:
         if get_runner().is_waiting_debugger_command():
             logging.debug("_check_issue_debugger_command: %s", cmd)
 
-            # tell VM the state we are seeing
+            # tell MainCPythonBackend the state we are seeing
             cmd.setdefault(
                 frame_id=self._last_progress_message.stack[-1].id,
                 breakpoints=self.get_effective_breakpoints(command),
@@ -63,11 +76,11 @@ class Debugger:
         return None
 
     def get_effective_breakpoints(self, command):
-        result = code.get_current_breakpoints()
+        result = editors.get_current_breakpoints()
 
         if command == "run_to_cursor":
             bp = self.get_run_to_cursor_breakpoint()
-            if bp != None:
+            if bp is not None:
                 filename, lineno = bp
                 result.setdefault(filename, set())
                 result[filename].add(lineno)
@@ -129,14 +142,14 @@ class Debugger:
             menu = tk.Menu(get_workbench())
             menu.add(
                 "command",
-                label=_("Run to cursor"),
+                label=tr("Run to cursor"),
                 command=lambda: self.check_issue_command("run_to_cursor"),
             )
             menu.add("separator")
             menu.add("command", label="Copy", command=create_edit_command_handler("<<Copy>>"))
             menu.add(
                 "command",
-                label=_("Select all"),
+                label=tr("Select all"),
                 command=create_edit_command_handler("<<SelectAll>>"),
             )
             self._editor_context_menu = menu
@@ -220,8 +233,9 @@ class SingleWindowDebugger(Debugger):
             )
 
     def handle_debugger_return(self, msg):
-        if self._last_frame_visualizer is not None and self._last_frame_visualizer.get_frame_id() == msg.get(
-            "frame_id"
+        if (
+            self._last_frame_visualizer is not None
+            and self._last_frame_visualizer.get_frame_id() == msg.get("frame_id")
         ):
             self._last_frame_visualizer.close()
 
@@ -536,7 +550,7 @@ class FrameVisualizer:
             if self._expression_box.winfo_ismapped():
                 dialog.title(self._expression_box.get_focused_text())
             else:
-                dialog.title("Function call at " + hex(self._frame_id))
+                dialog.title(tr("Function call at %s") % hex(self._frame_id))
 
             return dialog
 
@@ -627,6 +641,12 @@ class BaseExpressionBox:
         event = frame_info.event
 
         if frame_info.current_root_expression is not None:
+
+            if self._last_root_expression != frame_info.current_root_expression:
+                # can happen, eg. when focus jumps from the last expr in while body
+                # to while test expression
+                self.clear_debug_view()
+
             with open(frame_info.filename, "rb") as fp:
                 whole_source = fp.read()
 
@@ -801,6 +821,9 @@ class BaseExpressionBox:
         self.text["height"] = len(lines)
         self.text["width"] = max(map(len, lines))
 
+    def _set_position_make_visible(self, rel_x, rel_y):
+        raise NotImplementedError()
+
 
 class PlacedExpressionBox(BaseExpressionBox, tk.Text):
     def __init__(self, codeview):
@@ -848,6 +871,8 @@ class ToplevelExpressionBox(BaseExpressionBox, tk.Toplevel):
             raise RuntimeError("Should be used only on Mac")
 
         self.resizable(False, False)
+        if get_tk_version_info() >= (8, 6, 10) and running_on_mac_os():
+            self.wm_overrideredirect(1)
         self.wm_transient(codeview.winfo_toplevel())
         self.lift()
 
@@ -931,7 +956,7 @@ class DialogVisualizer(CommonDialog, FrameVisualizer):
         self._text_frame = CodeView(
             self._code_book, first_line_number=frame_info.firstlineno, font="EditorFont"
         )
-        self._code_book.add(self._text_frame, text="Source")
+        self._code_book.add(self._text_frame, text=tr("Source code"))
         self.main_pw.add(self._code_book, minsize=200)
         self._code_book.preferred_size_in_pw = 400
 
@@ -952,9 +977,9 @@ class DialogVisualizer(CommonDialog, FrameVisualizer):
 
     def _on_close(self):
         showinfo(
-            _("Can't close yet"),
-            _('Use "Stop" command if you want to cancel debugging'),
-            parent=None if running_on_mac_os() else self,
+            tr("Can't close yet"),
+            tr('Use "Stop" command if you want to cancel debugging'),
+            master=self,
         )
 
     def close(self, frame_id=None):
@@ -973,7 +998,7 @@ class FunctionCallDialog(DialogVisualizer):
         self._locals_book = ttk.Notebook(self.main_pw)
         self._locals_frame = VariablesFrame(self._locals_book)
         self._locals_book.preferred_size_in_pw = 200
-        self._locals_book.add(self._locals_frame, text="Local variables")
+        self._locals_book.add(self._locals_frame, text=tr("Local variables"))
         self.main_pw.add(self._locals_book, minsize=100)
 
     def _load_code(self, frame_info):
@@ -1005,8 +1030,8 @@ class StackView(ui_utils.TreeFrame):
         self.tree.column("function", width=120, anchor=tk.W, stretch=False)
         self.tree.column("location", width=450, anchor=tk.W, stretch=True)
 
-        self.tree.heading("function", text="Function", anchor=tk.W)
-        self.tree.heading("location", text="Location", anchor=tk.W)
+        self.tree.heading("function", text=tr("Function"), anchor=tk.W)
+        self.tree.heading("location", text=tr("Location"), anchor=tk.W)
 
         get_workbench().bind("DebuggerResponse", self._update_stack, True)
         get_workbench().bind("ToplevelResponse", lambda e=None: self._clear_tree(), True)
@@ -1081,7 +1106,8 @@ class ExceptionView(TextFrame):
     def _show_description(self):
         self.text.configure(foreground=get_syntax_options_for_tag("TEXT")["foreground"])
         self.text.direct_insert(
-            "end", "If last command raised an exception then this view will show the stacktrace."
+            "end",
+            tr("If last command raised an exception then this view will show the stacktrace."),
         )
 
     def set_exception(self, exception_lines_with_frame_info):
@@ -1251,7 +1277,7 @@ def run_preferred_debug_command():
 def load_plugin() -> None:
 
     global RESUME_COMMAND_CAPTION
-    RESUME_COMMAND_CAPTION = _("Resume")
+    RESUME_COMMAND_CAPTION = tr("Resume")
 
     if get_workbench().in_simple_mode():
         get_workbench().set_default("debugger.frames_in_separate_windows", False)
@@ -1267,7 +1293,7 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "runresume",
         "run",
-        _("Run / resume"),
+        tr("Run / resume"),
         _run_or_resume,
         caption=running.RUN_COMMAND_CAPTION,
         tester=_run_or_resume_enabled,
@@ -1282,9 +1308,9 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "debug_preferred",
         "run",
-        _("Debug current script"),
+        tr("Debug current script"),
         run_preferred_debug_command,
-        caption=_("Debug"),
+        caption=tr("Debug"),
         tester=_start_debug_enabled,
         group=10,
         image="debug-current-script",
@@ -1295,7 +1321,7 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "debug_nicer",
         "run",
-        _("Debug current script (nicer)"),
+        tr("Debug current script (nicer)"),
         lambda: _request_debug("Debug"),
         caption="Debug (nicer)",
         tester=_start_debug_enabled,
@@ -1307,7 +1333,7 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "debug_faster",
         "run",
-        _("Debug current script (faster)"),
+        tr("Debug current script (faster)"),
         lambda: _request_debug("FastDebug"),
         caption="Debug (faster)",
         tester=_start_debug_enabled,
@@ -1318,9 +1344,9 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "step_over",
         "run",
-        _("Step over"),
+        tr("Step over"),
         lambda: _issue_debugger_command("step_over"),
-        caption=_("Over"),
+        caption=tr("Over"),
         tester=lambda: _debugger_command_enabled("step_over"),
         default_sequence="<F6>",
         group=30,
@@ -1331,9 +1357,9 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "step_into",
         "run",
-        _("Step into"),
+        tr("Step into"),
         lambda: _issue_debugger_command("step_into"),
-        caption=_("Into"),
+        caption=tr("Into"),
         tester=lambda: _debugger_command_enabled("step_into"),
         default_sequence="<F7>",
         group=30,
@@ -1344,9 +1370,9 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "step_out",
         "run",
-        _("Step out"),
+        tr("Step out"),
         lambda: _issue_debugger_command("step_out"),
-        caption=_("Out"),
+        caption=tr("Out"),
         tester=lambda: _debugger_command_enabled("step_out"),
         group=30,
         image="step-out",
@@ -1369,7 +1395,7 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "run_to_cursor",
         "run",
-        _("Run to cursor"),
+        tr("Run to cursor"),
         lambda: _issue_debugger_command("run_to_cursor"),
         tester=lambda: _debugger_command_enabled("run_to_cursor"),
         default_sequence=select_sequence("<Control-F8>", "<Control-F8>"),
@@ -1381,16 +1407,16 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "step_back",
         "run",
-        _("Step back"),
+        tr("Step back"),
         lambda: _issue_debugger_command("step_back"),
-        caption=_("Back"),
+        caption=tr("Back"),
         tester=lambda: _debugger_command_enabled("step_back"),
         default_sequence=select_sequence("<Control-b>", "<Command-b>"),
         group=30,
     )
 
-    get_workbench().add_view(StackView, _("Stack"), "se")
-    get_workbench().add_view(ExceptionView, _("Exception"), "s")
+    get_workbench().add_view(StackView, tr("Stack"), "se")
+    get_workbench().add_view(ExceptionView, tr("Exception"), "s")
     get_workbench().bind("DebuggerResponse", _handle_debugger_progress, True)
     get_workbench().bind("ToplevelResponse", _handle_toplevel_response, True)
     get_workbench().bind("debugger_return_response", _handle_debugger_return, True)
