@@ -1,7 +1,5 @@
 import tkinter as tk
 
-from jedi import Script
-
 from thonny import get_workbench
 from thonny.ui_utils import control_is_pressed
 
@@ -18,6 +16,8 @@ def goto_definition(event):
     index_parts = index.split(".")
     line, column = int(index_parts[0]), int(index_parts[1])
     # TODO: find current editor filename
+    from jedi import Script
+
     script = Script(source, line=line, column=column, path="")
     defs = script.goto_definitions()
     if len(defs) > 0:
@@ -27,9 +27,7 @@ def goto_definition(event):
         if module_path and line is not None:
             get_workbench().get_editor_notebook().show_file(module_path, line)
         elif module_name == "" and line is not None:  # current editor
-            get_workbench().get_editor_notebook().get_current_editor().select_range(
-                line
-            )
+            get_workbench().get_editor_notebook().get_current_editor().select_range(line)
 
 
 def load_plugin() -> None:
