@@ -1,20 +1,37 @@
-import serial
-from thonny.plugins.micropython.connection import MicroPythonConnection, ConnectionFailedException
-import threading
-import time
-from serial.serialutil import SerialException
 import logging
 import platform
 import sys
+import threading
+import time
 from textwrap import dedent
 
+from thonny.plugins.micropython.bare_metal_backend import NORMAL_PROMPT, FIRST_RAW_PROMPT
+from thonny.common import ConnectionFailedException
+from thonny.plugins.micropython.connection import MicroPythonConnection
+
 
 class SerialConnection(MicroPythonConnection):
-    def __init__(self, port, baudrate, skip_reader=False):
+    def __init__(self, port, baudrate, dtr=None, rts=None, skip_reader=False):
+
+        import serial
+        from serial.serialutil import SerialException
+
         super().__init__()
 
         try:
-            self._serial = serial.Serial(port, baudrate=baudrate, timeout=None)
+            self._serial = serial.Serial(port=None, baudrate=baudrate, timeout=None, exclusive=True)
+            # Tweaking dtr and rts was proposed by
+            # https://github.com/thonny/thonny/pull/1187
+            # but in some cases it messes up communication.
+            # At the same time, in some cases it is required
+            # https://github.com/thonny/thonny/issues/1462
+            if dtr is not None:
+                self._serial.dtr = dtr
+            if rts is not None:
+                self._serial.rts = rts
+
+            self._serial.port = port
+            self._serial.open()
         except SerialException as error:
             err_str = str(error)
             if "FileNotFoundError" in err_str:
@@ -31,11 +48,10 @@ class SerialConnection(MicroPythonConnection):
                 (NB! This needs to be followed by reboot or logging out and logging in again!)"""
                 )
 
-            elif "PermissionError" in message:
+            elif "PermissionError" in message or "Could not exclusively lock" in message:
                 message += "\n\n" + dedent(
                     """\
-                If you have serial connection to the device from another program,
-                then disconnect it there."""
+                If you have serial connection to the device from another program, then disconnect it there first."""
                 )
 
             elif error.errno == 16:
@@ -49,15 +65,10 @@ class SerialConnection(MicroPythonConnection):
             self._reading_thread = threading.Thread(target=self._listen_serial, daemon=True)
             self._reading_thread.start()
 
-    def write(self, data, block_size=256, delay=0.01):
-        # delay and block size taken from rshell
-        # https://github.com/dhylands/rshell/blob/master/rshell/pyboard.py#L242
-        for i in range(0, len(data), block_size):
-            block = data[i : i + block_size]
-            # self._log_data(b"[" + block + b"]")
-            size = self._serial.write(block)
-            assert size == len(block)
-            time.sleep(delay)
+    def write(self, data):
+        size = self._serial.write(data)
+        # print(data.decode(), end="")
+        assert size == len(data)
         return len(data)
 
     def _listen_serial(self):
@@ -71,7 +82,6 @@ class SerialConnection(MicroPythonConnection):
                     # print("LISTEN EOFFFFFFFFFF")
                     break
                 data += self._serial.read_all()
-                self.num_bytes_received += len(data)
 
                 # don't publish incomplete utf-8 data
                 try:
@@ -89,7 +99,7 @@ class SerialConnection(MicroPythonConnection):
                         data = data[e.start :]
 
                 if to_be_published:
-                    self._read_queue.put(to_be_published)
+                    self._make_output_available(to_be_published)
 
         except Exception as e:
             self._error = str(e)
@@ -117,5 +127,28 @@ class SerialConnection(MicroPythonConnection):
                     logging.exception("Couldn't close serial")
 
 
+class DifficultSerialConnection(SerialConnection):
+    """For hardening the communication protocol"""
+
+    def _make_output_available(self, data, block=True):
+
+        # output prompts in parts
+        if FIRST_RAW_PROMPT in data or NORMAL_PROMPT in data:
+            if FIRST_RAW_PROMPT in data:
+                start = data.find(FIRST_RAW_PROMPT)
+                end = start + len(FIRST_RAW_PROMPT)
+            else:
+                start = data.find(NORMAL_PROMPT)
+                end = start + len(NORMAL_PROMPT)
+
+            super()._make_output_available(data[: start + 1], block=block)
+            time.sleep(0.1)
+            super()._make_output_available(data[start + 1 : end - 1], block=block)
+            time.sleep(0.1)
+            super()._make_output_available(data[end - 1 :], block=block)
+        else:
+            super()._make_output_available(data, block=block)
+
+
 def debug(*args, file=sys.stderr):
     print(*args, file=file)
