@@ -1,19 +1,19 @@
 import ast
-import json
 import os.path
 import tkinter as tk
 from datetime import datetime
 from tkinter import ttk
 
-from thonny import codeview, get_workbench, ui_utils, THONNY_USER_DIR
+from thonny import THONNY_USER_DIR, codeview, get_workbench, ui_utils
 from thonny.base_file_browser import BaseLocalFileBrowser
+from thonny.languages import tr
 from thonny.plugins.coloring import SyntaxColorer
-from thonny.ui_utils import lookup_style_option, CommonDialog
+from thonny.ui_utils import CommonDialog, lookup_style_option
 
 
 class ReplayWindow(CommonDialog):
-    def __init__(self):
-        super().__init__(get_workbench(), background=lookup_style_option("TFrame", "background"))
+    def __init__(self, master):
+        super().__init__(master, background=lookup_style_option("TFrame", "background"))
         ui_utils.set_zoomed(self, True)
 
         self.main_pw = ReplayerPanedWindow(self, orient=tk.HORIZONTAL, sashwidth=10)
@@ -64,7 +64,11 @@ class ReplayerFileBrowser(BaseLocalFileBrowser):
         # self.save_current_folder()
         path = self.get_selected_path()
         if path:
-            self.log_frame.load_log(path)
+            kind = self.get_selected_kind()
+            if kind == "dir":
+                self.focus_into(path)
+            else:
+                self.log_frame.load_log(path)
 
         return "break"  # avoid default action of opening the node
 
@@ -107,12 +111,17 @@ class LogFrame(ui_utils.TreeFrame):
         self.editor_notebook.reset()
         self.shell.reset()
 
+        import json
+
         with open(filename, encoding="UTF-8") as f:
             events = json.load(f)
             last_event_time = None
             for event in events:
                 node_id = self.tree.insert("", "end")
                 self.tree.set(node_id, "desc", event["sequence"])
+                if len(event["time"]) == 19:
+                    # 0 fraction may have been skipped
+                    event["time"] += ".0"
                 event_time = datetime.strptime(event["time"], "%Y-%m-%dT%H:%M:%S.%f")
                 if last_event_time:
                     delta = event_time - last_event_time
@@ -347,11 +356,11 @@ class ReplayerPanedWindow(tk.PanedWindow):
 
 def load_plugin() -> None:
     def open_replayer():
-        win = ReplayWindow()
+        win = ReplayWindow(get_workbench())
         ui_utils.show_dialog(win)
 
     get_workbench().set_default("tools.replayer_last_browser_folder", None)
     if get_workbench().get_ui_mode() == "expert":
         get_workbench().add_command(
-            "open_replayer", "tools", _("Open replayer..."), open_replayer, group=110
+            "open_replayer", "tools", tr("Open replayer..."), open_replayer, group=110
         )
