@@ -1,17 +1,17 @@
-import ast
 import builtins
+import os.path
 import re
 import token
-import tokenize
-import os.path
 
-from thonny.assistance import ErrorHelper, Suggestion, name_similarity, add_error_helper
 from thonny import assistance
-from thonny.misc_utils import running_on_linux, running_on_windows
+from thonny.assistance import ErrorHelper, Suggestion, add_error_helper, name_similarity
+from thonny.misc_utils import running_on_windows
 
 
 class SyntaxErrorHelper(ErrorHelper):
     def __init__(self, error_info):
+        import tokenize
+
         super().__init__(error_info)
 
         self.tokens = []
@@ -66,6 +66,8 @@ class SyntaxErrorHelper(ErrorHelper):
                 self.suggestions = [self._sug_missing_or_misplaced_colon()]
 
     def _sug_missing_or_misplaced_colon(self):
+        import tokenize
+
         i = 0
         title = "Did you forget the colon?"
         relevance = 0
@@ -379,6 +381,8 @@ class NameErrorHelper(ErrorHelper):
         return Suggestion("missing-import", "Did you forget to import it?", body, relevance)
 
     def _sug_local_from_global(self):
+        import ast
+
         relevance = 0
         body = None
 
@@ -496,7 +500,7 @@ class AttributeErrorHelper(ErrorHelper):
         elif self.type_name == "dict":
             goal = "number of entries"
         else:
-            return
+            return None
 
         return Suggestion(
             "wrong-attribute-instead-of-len",
