@@ -1,31 +1,41 @@
 # -*- coding: utf-8 -*-
 
-from _tkinter import TclError
 import logging
 import os.path
 import re
-from tkinter import ttk
+import tkinter as tk
 import traceback
+from tkinter import ttk
 
-from thonny import get_runner, get_workbench, memory, roughparse, ui_utils, running
-from thonny.codeview import PythonText, get_syntax_options_for_tag
-from thonny.common import InlineCommand, ToplevelCommand, ToplevelResponse
+from _tkinter import TclError
+
+from thonny import get_runner, get_workbench, memory, roughparse, running, ui_utils
+from thonny.codeview import get_syntax_options_for_tag, perform_python_return, SyntaxText
+from thonny.common import (
+    OBJECT_LINK_END,
+    OBJECT_LINK_START,
+    InlineCommand,
+    ToplevelCommand,
+    ToplevelResponse,
+)
+from thonny.languages import tr
 from thonny.misc_utils import construct_cmd_line, parse_cmd_line, running_on_mac_os, shorten_repr
-from thonny.tktextext import index2line, TextFrame, TweakableText
+from thonny.running import EDITOR_CONTENT_TOKEN
+from thonny.tktextext import TextFrame, TweakableText, index2line
 from thonny.ui_utils import (
+    CommonDialog,
     EnhancedTextWithLogging,
-    scrollbar_style,
-    select_sequence,
     TextMenu,
     create_tooltip,
-    show_dialog,
     lookup_style_option,
-    CommonDialog,
+    scrollbar_style,
+    select_sequence,
+    show_dialog,
     tr_btn,
+    ems_to_pixels,
 )
-import tkinter as tk
-import webbrowser
-from thonny.running import EDITOR_CONTENT_TOKEN
+
+logger = logging.getLogger(__name__)
 
 _CLEAR_SHELL_DEFAULT_SEQ = select_sequence("<Control-l>", "<Command-k>")
 
@@ -33,7 +43,7 @@ _CLEAR_SHELL_DEFAULT_SEQ = select_sequence("<Control-l>", "<Command-k>")
 OUTPUT_SPLIT_REGEX = re.compile(r"(\x1B\[[0-?]*[ -/]*[@-~]|[\a\b\r])")
 NUMBER_SPLIT_REGEX = re.compile(r"((?<!\w)[-+]?[0-9]*\.?[0-9]+\b)")
 SIMPLE_URL_SPLIT_REGEX = re.compile(
-    r"(https?:\/\/[\w\/.:\-\?#=%]+[\w\/]|data:image\/[a-z]+;base64,[A-Za-z0-9\/=\+]+)"
+    r"(https?:\/\/[\w\/.:\-\?#=%&]+[\w\/]|data:image\/[a-z]+;base64,[A-Za-z0-9\/=\+]+)"
 )
 
 INT_REGEX = re.compile(r"\d+")
@@ -55,7 +65,7 @@ class ShellView(tk.PanedWindow):
         super().__init__(
             master,
             orient="horizontal",
-            sashwidth=lookup_style_option("Sash", "sashthickness", 4),
+            sashwidth=lookup_style_option("Sash", "sashthickness", ems_to_pixels(0.6)),
             background=lookup_style_option("TPanedWindow", "background"),
             borderwidth=0,
         )
@@ -70,7 +80,7 @@ class ShellView(tk.PanedWindow):
         get_workbench().add_command(
             "clear_shell",
             "edit",
-            _("Clear shell"),
+            tr("Clear shell"),
             self.clear_shell,
             default_sequence=_CLEAR_SHELL_DEFAULT_SEQ,
             group=200,
@@ -79,6 +89,7 @@ class ShellView(tk.PanedWindow):
         get_workbench().set_default("shell.max_lines", 1000)
         get_workbench().set_default("shell.squeeze_threshold", 1000)
         get_workbench().set_default("shell.tty_mode", True)
+        get_workbench().set_default("shell.auto_inspect_values", True)
 
         self.text = ShellText(
             main_frame,
@@ -124,7 +135,7 @@ class ShellView(tk.PanedWindow):
         get_workbench().add_command(
             "toggle_plotter",
             "view",
-            _("Plotter"),
+            tr("Plotter"),
             self.toggle_plotter,
             can_toggle,
             flag_name="view.show_plotter",
@@ -133,6 +144,9 @@ class ShellView(tk.PanedWindow):
 
         self.update_plotter_visibility(True)
 
+    def set_ignore_program_output(self, value):
+        self.text._ignore_program_output = value
+
     def toggle_plotter(self):
         self.plotter_visibility_var.set(not self.plotter_visibility_var.get())
         self.update_plotter_visibility()
@@ -179,6 +193,9 @@ class ShellView(tk.PanedWindow):
     def print_error(self, txt):
         self.text._insert_text_directly(txt, ("io", "stderr"))
 
+    def insert_command_link(self, txt, handler):
+        self.text._insert_command_link(txt, handler)
+
     def focus_set(self):
         self.text.focus_set()
 
@@ -249,7 +266,7 @@ class ShellMenu(TextMenu):
 
     def add_extra_items(self):
         self.add_separator()
-        self.add_command(label=_("Clear"), command=self.text._clear_shell)
+        self.add_command(label=tr("Clear"), command=self.text._clear_shell)
 
         def toggle_from_menu():
             # I don't like that Tk menu toggles checbutton variable
@@ -262,7 +279,7 @@ class ShellMenu(TextMenu):
             self.view.toggle_plotter()
 
         self.add_checkbutton(
-            label=_("Show Plotter"),
+            label=tr("Show Plotter"),
             command=toggle_from_menu,
             variable=self.view.plotter_visibility_var,
         )
@@ -271,11 +288,13 @@ class ShellMenu(TextMenu):
         return not self.text.selection_is_writable()
 
 
-class BaseShellText(EnhancedTextWithLogging, PythonText):
+class BaseShellText(EnhancedTextWithLogging, SyntaxText):
     """Passive version of ShellText. Used also for preview"""
 
     def __init__(self, master, view=None, cnf={}, **kw):
         self.view = view
+        self._ignore_program_output = False
+        self._link_handler_count = 0
         kw["tabstyle"] = "wordprocessor"
         super().__init__(master, cnf, **kw)
 
@@ -350,6 +369,9 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         self.mark_set("output_insert", "end-1c")
         self.mark_gravity("output_insert", tk.RIGHT)
 
+        self.mark_set("command_io_start", "1.0")
+        self.mark_gravity("command_io_start", "left")
+
         self.active_object_tags = set()
 
         self.update_tabs()
@@ -362,6 +384,9 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         self.tag_raise("intense_italic_io")
         self.tag_raise("sel")
 
+    def is_python_text(self):
+        return True
+
     def submit_command(self, cmd_line, tags):
         # assert get_runner().is_waiting_toplevel_command()
         self.delete("input_start", "end")
@@ -379,6 +404,9 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         self.see("end")
 
     def _handle_program_output(self, msg):
+        if self._ignore_program_output:
+            # This output will be handled elsewhere
+            return
         # Discard but not too often, as toplevel response will discard anyway
         if int(float(self.index("end"))) > get_workbench().get_option("shell.max_lines") + 100:
             self._discard_old_content()
@@ -391,6 +419,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
             self.tag_add("before_io", "output_insert -1 line linestart")
 
         self._update_visible_io(None)
+        self._render_object_links("end")
 
     def _handle_toplevel_response(self, msg: ToplevelResponse) -> None:
         if msg.get("error"):
@@ -407,6 +436,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
             if preceding.strip() and not preceding.endswith("\n"):
                 self._insert_text_directly("\n")
             self._insert_text_directly(welcome_text, ("welcome",))
+            self.see("end")
 
         if "value_info" in msg:
             num_stripped_question_marks = getattr(msg, "num_stripped_question_marks", 0)
@@ -535,9 +565,14 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
             for url_match in SIMPLE_URL_SPLIT_REGEX.finditer(data):
                 non_url_length -= url_match.end() - url_match.start()
 
-            if non_url_length > self._get_squeeze_threshold() and "\n" not in data:
+            if (
+                non_url_length > self._get_squeeze_threshold()
+                and "\n" not in data
+                and not (data.startswith(OBJECT_LINK_START))
+            ):
                 self._io_cursor_offset = 0  # ignore the effect of preceding \r and \b
-                button_text = data[:40] + " …"
+                actual_text = self._remove_object_link_markers(data)
+                button_text = actual_text[:70] + " …"
                 btn = tk.Label(
                     self,
                     text=button_text,
@@ -548,7 +583,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
                     font="IOFont",
                 )
                 btn.bind("<1>", lambda e: self._show_squeezed_text(btn), True)
-                btn.contained_text = data
+                btn.contained_text = actual_text
                 btn.tags = tags
                 self._squeeze_buttons.add(btn)
                 create_tooltip(btn, "%d characters squeezed. " % len(data) + "Click for details.")
@@ -587,7 +622,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
                     self._io_cursor_offset = 0
                 else:
                     # offset becomes closer to 0
-                    self._io_cursor_offset + overwrite_len
+                    self._io_cursor_offset += overwrite_len
 
             elif self._io_cursor_offset > 0:
                 # insert spaces before actual data
@@ -601,6 +636,62 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
 
         self._applied_io_events.append((original_data, stream_name))
 
+    def _remove_object_link_markers(self, data):
+        end_pos = data.find(OBJECT_LINK_END)
+        if end_pos < 0:
+            return data
+
+        at_pos = data.rfind("@", 0, end_pos)
+        if at_pos < 0:
+            return data
+
+        data = data[:at_pos] + data[end_pos + len(OBJECT_LINK_END) :]
+        return data.replace(OBJECT_LINK_START, "")
+
+    def _render_object_links(self, start):
+        closer_start = self.search(
+            OBJECT_LINK_END, index=start, stopindex="command_io_start", backwards=True, elide=False
+        )
+        if not closer_start:
+            return
+
+        id_start = self.search(
+            "@", index=closer_start, stopindex="command_io_start", backwards=True
+        )
+        if not id_start:
+            logger.warning("Can't find object id")
+            return
+
+        id_str = self.get(id_start, closer_start)[1:]
+        try:
+            int(id_str)
+        except ValueError:
+            logger.warning("Can't parse object id: " + id_str)
+            return
+
+        self.direct_delete(id_start, "%s + %d chars" % (closer_start, len(OBJECT_LINK_END)))
+
+        opener_start = self.search(
+            OBJECT_LINK_START, index=id_start, stopindex="command_io_start", backwards=True
+        )
+        value_start = "%s + %d chars" % (opener_start, len(OBJECT_LINK_START))
+
+        self.tag_remove("io", value_start, closer_start)
+        self.tag_add("value", value_start, closer_start)
+        self.tag_add(id_str, value_start, closer_start)
+
+        if get_workbench().in_heap_mode():
+            id_repr = memory.format_object_id(int(id_str))
+            self.direct_delete(value_start, id_start)
+            self.direct_insert(value_start, id_repr, ("value", id_str))
+
+        self.direct_delete(opener_start, value_start)
+
+        self._render_object_links(opener_start)
+
+        if get_workbench().get_option("shell.auto_inspect_values"):
+            get_workbench().event_generate("ObjectSelect", object_id=int(id_str))
+
     def _show_squeezed_text(self, button):
         dlg = SqueezedTextDialog(self, button)
         show_dialog(dlg)
@@ -608,14 +699,14 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
     def _change_io_cursor_offset_csi(self, marker):
         ints = re.findall(INT_REGEX, marker)
         if len(ints) != 1:
-            logging.warn("bad CSI cursor positioning: %s", marker)
+            logging.warning("bad CSI cursor positioning: %s", marker)
             # do nothing
             return
 
         try:
             delta = int(ints[0])
         except ValueError:
-            logging.warn("bad CSI cursor positioning: %s", marker)
+            logging.warning("bad CSI cursor positioning: %s", marker)
             return
 
         if marker.endswith("D"):
@@ -836,6 +927,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
             # "\n" + "═"*200 + "\n",
             ("magic", "restart_line"),
         )
+        self.see("end")
 
     def intercept_insert(self, index, txt, tags=()):
         # pylint: disable=arguments-differ
@@ -916,7 +1008,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
                     EnhancedTextWithLogging.perform_return(self, event)
                 else:
                     # Allow auto-indent
-                    PythonText.perform_return(self, event)
+                    perform_python_return(self, event)
 
             self._try_submit_input()
 
@@ -933,6 +1025,13 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         except Exception:
             return False
 
+    def _insert_command_link(self, txt, handler):
+        self._link_handler_count += 1
+        command_tag = "link_handler_%s" % self._link_handler_count
+
+        self.direct_insert("output_insert", txt, ("io_hyperlink", command_tag))
+        self.tag_bind(command_tag, "<1>", handler)
+
     def _insert_text_directly(self, txt, tags=()):
         def _insert(txt, tags):
             if txt != "":
@@ -965,7 +1064,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         tags = tuple(tags)
 
         # Make stacktrace clickable
-        if "stderr" in tags or "error" in tags:
+        if "stderr" in tags or "error" in tags or ("File" in txt and "line" in txt):
             # show lines pointing to source lines as hyperlinks
             for line in txt.splitlines(True):
                 parts = re.split(r"(File .* line \d+.*)$", line, maxsplit=1)
@@ -1267,6 +1366,10 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         end_index = self.index("output_end")
         self._clear_content(end_index)
 
+    def _on_backend_restart(self, event=None):
+        # make sure dead values are not clickable anymore
+        self.tag_remove("value", "0.1", "end")
+
     def compute_smart_home_destination_index(self):
         """Is used by EnhancedText"""
 
@@ -1276,13 +1379,29 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         else:
             return super().compute_smart_home_destination_index()
 
-    def _hyperlink_enter(self, event):
-        self.config(cursor="hand2")
+    def _value_click(self, event):
+        pos = "@%d,%d" % (event.x, event.y)
+        tags = self.tag_names(pos)
+        for tag in tags:
+            if tag.isnumeric():
+                get_workbench().show_view("ObjectInspector", set_focus=False)
+                get_workbench().update_idletasks()
+                get_workbench().event_generate("ObjectSelect", object_id=int(tag))
 
-    def _hyperlink_leave(self, event):
-        self.config(cursor="")
+    def _value_mouse_up(self, event):
+        pos = "@%d,%d" % (event.x, event.y)
+        rng = self.tag_prevrange("value", pos)
+        if not rng:
+            return
+
+        # select whole value unless user has started a partial selection
+        if not self.tag_nextrange("sel", rng[0], rng[1]):
+            self.tag_remove("sel", "1.0", "end")
+            self.tag_add("sel", rng[0], rng[1])
 
     def _handle_hyperlink(self, event):
+        import webbrowser
+
         try:
             line = self.get("insert linestart", "insert lineend")
             # Python stacktrace
@@ -1314,11 +1433,6 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
             traceback.print_exc()
 
     def _show_user_exception(self, user_exception):
-        fr_tr = user_exception.get("friendly_traceback", None)
-        if fr_tr is not None:
-            self._format_friendly_traceback(fr_tr)
-            return
-
         for line, frame_id, *_ in user_exception["items"]:
 
             tags = ("io", "stderr")
@@ -1335,13 +1449,6 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
 
             self._insert_text_directly(line, tags)
 
-    def _format_friendly_traceback(self, s):
-        tags = ("io", "stderr")
-
-        lines = s.splitlines(True)
-        for line in lines:
-            self._insert_text_directly(line, tags)
-
     def _discard_old_content(self):
         max_lines = max(get_workbench().get_option("shell.max_lines"), 0)
         proposed_cut = self.index("end -%d lines linestart" % max_lines)
@@ -1358,16 +1465,28 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
     def _clear_content(self, cut_idx):
         proposed_cut_float = float(cut_idx)
         for btn in list(self._squeeze_buttons):
-            btn_pos = float(self.index(btn))
-            if btn_pos < proposed_cut_float:
+            idx = self.index(btn)
+            if idx is None or float(idx) < proposed_cut_float:
                 self._squeeze_buttons.remove(btn)
                 # looks like the widgets are not fully GC-d.
                 # At least avoid leaking big chunks of texts
-                btn.contained_text = None
-                btn.destroy()
+                try:
+                    btn.contained_text = None
+                    btn.destroy()
+                except:
+                    logger.warning("Could not destroy a squeeze button")
 
         self.direct_delete("0.1", cut_idx)
 
+    def _on_mouse_move(self, event=None):
+        tags = self.tag_names("@%d,%d" % (event.x, event.y))
+        if "value" in tags or "io_hyperlink" in tags:
+            if self.cget("cursor") != "hand2":
+                self.config(cursor="hand2")
+        else:
+            if self.cget("cursor"):
+                self.config(cursor="")
+
     def _invalidate_current_data(self):
         """
         Grayes out input & output displayed so far
@@ -1405,20 +1524,31 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         if ui_utils.get_tk_version_info() >= (8, 6, 6):
             self.tag_configure("io", lmargincolor=get_syntax_options_for_tag("TEXT")["background"])
 
+    def _hide_trailing_output(self, msg):
+        pos = self.search(msg.text, index="end", backwards=True)
+        if pos:
+            end_pos = self.index("%s + %d chars" % (pos, len(msg.text)))
+            if end_pos == self.index("output_end"):
+                self.direct_delete(pos, end_pos)
+
 
 class ShellText(BaseShellText):
     def __init__(self, master, view, cnf={}, **kw):
         super().__init__(master, view, cnf=cnf, **kw)
         self.bindtags(self.bindtags() + ("ShellText",))
 
+        self.tag_bind("value", "<1>", self._value_click)
+        self.tag_bind("value", "<ButtonRelease-1>", self._value_mouse_up)
         self.tag_bind("io_hyperlink", "<ButtonRelease-1>", self._handle_hyperlink)
-        self.tag_bind("io_hyperlink", "<Enter>", self._hyperlink_enter)
-        self.tag_bind("io_hyperlink", "<Leave>", self._hyperlink_leave)
+
+        self.bind("<Motion>", self._on_mouse_move, True)
 
         get_workbench().bind("InputRequest", self._handle_input_request, True)
         get_workbench().bind("ProgramOutput", self._handle_program_output, True)
         get_workbench().bind("ToplevelResponse", self._handle_toplevel_response, True)
         get_workbench().bind("DebuggerResponse", self._handle_fancy_debugger_progress, True)
+        get_workbench().bind("BackendRestart", self._on_backend_restart, True)
+        get_workbench().bind("HideTrailingOutput", self._hide_trailing_output, True)
 
 
 class SqueezedTextDialog(CommonDialog):
@@ -1439,7 +1569,7 @@ class SqueezedTextDialog(CommonDialog):
 
         explanation_label = ttk.Label(
             mainframe,
-            text=_(
+            text=tr(
                 "For performance reasons, Shell avoids showing "
                 + "very long lines in full (see Tools => Options => Shell).\n"
                 + "Here you can interact with the original text fragment."
@@ -1450,7 +1580,7 @@ class SqueezedTextDialog(CommonDialog):
         self._wrap_var = tk.BooleanVar(False)
         self.wrap_checkbox = ttk.Checkbutton(
             mainframe,
-            text=_("Wrap text (may be slow)"),
+            text=tr("Wrap text (may be slow)"),
             variable=self._wrap_var,
             onvalue=True,
             offvalue=False,
@@ -1492,7 +1622,7 @@ class SqueezedTextDialog(CommonDialog):
 
         self.bind("<Escape>", self._on_close, True)
         self.protocol("WM_DELETE_WINDOW", self._on_close)
-        self.title(_("Squeezed text (%d characters)") % len(self.content))
+        self.title(tr("Squeezed text (%d characters)") % len(self.content))
 
     def _on_wrap_changed(self):
         if self._wrap_var.get():
@@ -1661,9 +1791,9 @@ class PlotterCanvas(tk.Canvas):
         self.delete("info")
         if segment_count == 0:
             info_text = (
-                _("Plotter visualizes series of\n" + "numbers printed to the Shell.")
+                tr("Plotter visualizes series of\n" + "numbers printed to the Shell.")
                 + "\n\n"
-                + _("See Help for details.")
+                + tr("See Help for details.")
             )
 
             self.create_text_with_background(
@@ -1778,8 +1908,8 @@ class PlotterCanvas(tk.Canvas):
         if not segments_by_color:
             return
 
-        range_start = 9999999999
-        range_end = -9999999999
+        range_start = 2 ** 15
+        range_end = -(2 ** 15)
 
         # if new block is using 3/4 of the width,
         # then don't consider old block's values anymore
