@@ -6,9 +6,9 @@ import time
 import tkinter as tk
 import traceback
 from logging import exception
-from tkinter import TclError, messagebox
+from tkinter import TclError
 from tkinter import font as tkfont
-from tkinter import ttk
+from tkinter import messagebox, ttk
 
 
 class TweakableText(tk.Text):
@@ -53,7 +53,13 @@ class TweakableText(tk.Text):
                 and operation in ["delete", "index", "get"]
                 and args in [("sel.first", "sel.last"), ("sel.first",)]
             ):
-
+                pass
+            # Don't worry about hitting ends of undo/redo stacks
+            elif (
+                operation == "edit"
+                and args in [("undo",), ("redo",)]
+                and str(e).lower() == "nothing to " + args[0]
+            ):
                 pass
             else:
                 exception(
@@ -75,12 +81,12 @@ class TweakableText(tk.Text):
 
     def set_insertwidth(self, new_width):
         """Change cursor width
-        
+
         NB! Need to be careful with setting text["insertwidth"]!
         My first straightforward solution caused unexplainable
         infinite loop of insertions and deletions in the text
         (Repro: insert a line and a word, select that word and then do Ctrl-Z).
-        
+
         This solution seems safe but be careful!
         """
         if self._suppress_events:
@@ -163,23 +169,14 @@ class TweakableText(tk.Text):
 
 
 class EnhancedText(TweakableText):
-    """Text widget with extra navigation and editing aids. 
+    """Text widget with extra navigation and editing aids.
     Provides more comfortable deletion, indentation and deindentation,
     and undo handling. Not specific to Python code.
-    
+
     Most of the code is adapted from idlelib.EditorWindow.
     """
 
-    def __init__(
-        self,
-        master=None,
-        style="Text",
-        tag_current_line=False,
-        indent_with_tabs=False,
-        replace_tabs=False,
-        cnf={},
-        **kw
-    ):
+    def __init__(self, master=None, style="Text", tag_current_line=False, cnf={}, **kw):
         # Parent class shouldn't autoseparate
         # TODO: take client provided autoseparators value into account
         kw["autoseparators"] = False
@@ -187,14 +184,13 @@ class EnhancedText(TweakableText):
         self._original_options = kw.copy()
 
         super().__init__(master=master, cnf=cnf, **kw)
-        self.tabwidth = 8  # See comments in idlelib.editor.EditorWindow
+        self.tabwidth = 4
         self.indent_width = 4
-        self.indent_with_tabs = indent_with_tabs
-        self.replace_tabs = replace_tabs
 
         self._last_event_kind = None
         self._last_key_time = None
 
+        self._bind_keypad()
         self._bind_editing_aids()
         self._bind_movement_aids()
         self._bind_selection_aids()
@@ -215,6 +211,9 @@ class EnhancedText(TweakableText):
             self.bind("<<TextChange>>", self._tag_current_line, True)
             self._tag_current_line()
 
+    def should_indent_with_tabs(self):
+        return True
+
     def _bind_mouse_aids(self):
         if _running_on_mac():
             self.bind("<Button-2>", self.on_secondary_click)
@@ -249,6 +248,31 @@ class EnhancedText(TweakableText):
         if platform.system() == "Windows":
             self.bind("<KeyPress>", self._insert_untypable_characters_on_windows, True)
 
+    def _bind_keypad(self):
+        """Remap keypad movement events to non-keypad equivalents"""
+        # https://github.com/thonny/thonny/issues/1106
+        kmap = {
+            "<KP_Left>": "<Left>",
+            "<KP_Right>": "<Right>",
+            "<KP_Up>": "<Up>",
+            "<KP_Down>": "<Down>",
+            "<KP_Home>": "<Home>",
+            "<KP_End>": "<End>",
+            "<KP_Next>": "<Next>",
+            "<KP_Prior>": "<Prior>",
+            "<KP_Enter>": "<Return>",
+        }
+        for from_key in kmap:
+
+            def mfunc(event, key=from_key):
+                self.event_generate(kmap[key], **{"state": event.state})
+                return "break"
+
+            try:
+                self.bind(from_key, mfunc)
+            except TclError:
+                pass
+
     def _bind_movement_aids(self):
         self.bind("<Home>", self.perform_smart_home, True)
         self.bind("<Left>", self.move_to_edge_if_selection(0), True)
@@ -365,7 +389,12 @@ class EnhancedText(TweakableText):
     def perform_midline_tab(self, event=None):
         "autocompleter can put its magic here"
         # by default
-        return self.perform_smart_tab(event)
+        return self.perform_dumb_tab(event)
+
+    def perform_dumb_tab(self, event=None):
+        self._log_keypress_for_undo(event)
+        self.insert("insert", "\t")
+        return "break"
 
     def perform_smart_tab(self, event=None):
         self._log_keypress_for_undo(event)
@@ -390,7 +419,7 @@ class EnhancedText(TweakableText):
             self._reindent_to(effective + self.indent_width)
         else:
             # tab to the next 'stop' within or to right of line's text:
-            if self.indent_with_tabs:
+            if self.should_indent_with_tabs():
                 pad = "\t"
             else:
                 effective = len(prefix.expandtabs(self.tabwidth))
@@ -612,7 +641,7 @@ class EnhancedText(TweakableText):
 
     def _make_blanks(self, n):
         # Make string that displays as n leading blanks.
-        if self.indent_with_tabs:
+        if self.should_indent_with_tabs():
             ntabs, nspaces = divmod(n, self.tabwidth)
             return "\t" * ntabs + " " * nspaces
         else:
@@ -715,29 +744,8 @@ class EnhancedText(TweakableText):
         super().destroy()
 
     def direct_insert(self, index, chars, tags=None, **kw):
-        chars = self.check_convert_tabs_to_spaces(chars)
         super().direct_insert(index, chars, tags, **kw)
 
-    def check_convert_tabs_to_spaces(self, chars):
-        if not self.replace_tabs:
-            return chars
-        tab_count = chars.count("\t")
-        if tab_count == 0:
-            return chars
-        else:
-
-            if messagebox.askyesno(
-                "Convert tabs to spaces?",
-                "Thonny (according to Python recommendation) uses spaces for indentation, "
-                + "but the text you are about to insert/open contains %d tab characters. "
-                % tab_count
-                + "To avoid confusion, it's better to convert them into spaces (unless you know they should be kept as tabs).\n\n"
-                + "Do you want me to replace each tab with %d spaces?\n\n" % self.indent_width,
-            ):
-                return chars.expandtabs(self.indent_width)
-            else:
-                return chars
-
 
 class TextFrame(ttk.Frame):
     "Decorates text with scrollbars, listens for theme changes"
@@ -890,7 +898,7 @@ class EnhancedTextFrame(TextFrame):
             self._gutter.configure(height=text_options["height"])
 
         self._gutter_is_gridded = False
-        self._gutter.bind("<Double-Button-1>", self.on_gutter_double_click, True),
+        self._gutter.bind("<Double-Button-1>", self.on_gutter_double_click, True)
         self._gutter.bind("<ButtonRelease-1>", self.on_gutter_click, True)
         self._gutter.bind("<Button-1>", self.on_gutter_click, True)
         self._gutter.bind("<Button1-Motion>", self.on_gutter_motion, True)
@@ -1083,6 +1091,7 @@ class EnhancedTextFrame(TextFrame):
                 min(gutter_selection_start, linepos), max(gutter_selection_start - 1, linepos - 1)
             )
             self.text.mark_set("insert", "%s.0" % linepos)
+            self.text.focus_set()
         except tk.TclError:
             exception("on_gutter_motion")
 
