@@ -12,6 +12,11 @@ _builtinlist = [
     str(name) for name in dir(builtins) if not name.startswith("_") and name not in keyword.kwlist
 ]
 
+# Not really built-ins, but special names nevertheless
+_builtinlist.append("self")
+_builtinlist.append("cls")
+
+
 # TODO: move builtin handling to global-local
 BUILTIN = r"([^.'\"\\#]\b|^)" + matches_any("builtin", _builtinlist) + r"\b"
 NUMBER = matches_any("number", [r"\b(\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?"])
@@ -38,3 +43,5 @@ STRING_OPEN = matches_any("open_string", [SQSTRING_OPEN, DQSTRING_OPEN])
 STRING_CLOSED = matches_any("string", [SQSTRING_CLOSED, DQSTRING_CLOSED])
 STRING3_DELIMITER = matches_any("DELIMITER3", [SQ3DELIMITER, DQ3DELIMITER])
 STRING3 = matches_any("string3", [DQ3STRING, SQ3STRING])
+
+TAB = matches_any("tab", "\t")
