@@ -9,52 +9,42 @@ import pkgutil
 import platform
 import queue
 import re
+import shutil
 import socket
 import sys
 import tkinter as tk
 import tkinter.font as tk_font
 import traceback
-import webbrowser
-import gettext
 from threading import Thread
-from tkinter import ttk, messagebox
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    List,
-    Optional,  # pylint: disable=unused-import
-    Sequence,
-    Set,  # pylint: disable=unused-import
-    Tuple,  # pylint: disable=unused-import
-    Type,
-    Union,
-    cast,
-)  # pylint: disable=unused-import
+from tkinter import messagebox, ttk
+from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, Type, Union, cast
 from warnings import warn
 
 import thonny
 from thonny import (
     THONNY_USER_DIR,
-    get_runner,
-    running,
-    ui_utils,
     assistance,
-    languages,
+    get_runner,
     get_shell,
     is_portable,
+    languages,
+    running,
+    ui_utils,
 )
-from thonny.code import EditorNotebook
 from thonny.common import Record, UserError, normpath_with_actual_case
 from thonny.config import try_load_configuration
 from thonny.config_ui import ConfigurationDialog
+from thonny.editors import EditorNotebook
+from thonny.languages import tr
 from thonny.misc_utils import (
+    copy_to_clipboard,
     running_on_linux,
     running_on_mac_os,
-    running_on_windows,
     running_on_rpi,
-    copy_to_clipboard,
+    running_on_windows,
 )
+from thonny.plugins.microbit import MicrobitFlashingDialog
+from thonny.plugins.micropython.uf2dialog import Uf2FlashingDialog
 from thonny.running import BackendProxy, Runner
 from thonny.shell import ShellView
 from thonny.ui_utils import (
@@ -66,8 +56,12 @@ from thonny.ui_utils import (
     register_latin_shortcut,
     select_sequence,
     sequence_to_accelerator,
+    caps_lock_is_on,
+    shift_is_pressed,
 )
 
+logger = logging.getLogger(__name__)
+
 SERVER_SUCCESS = "OK"
 SIMPLE_MODE_VIEWS = ["ShellView"]
 
@@ -121,6 +115,7 @@ class Workbench(tk.Tk):
 
     def __init__(self) -> None:
         thonny._workbench = self
+        self.ready = False
         self._closing = False
         self._destroyed = False
         self._lost_focus = False
@@ -128,10 +123,12 @@ class Workbench(tk.Tk):
         self.initializing = True
 
         self._init_configuration()
+        self._tweak_environment()
         self._check_init_server_loop()
 
         tk.Tk.__init__(self, className="Thonny")
         tk.Tk.report_callback_exception = self._on_tk_exception  # type: ignore
+        ui_utils.add_messagebox_parent_checker()
         self._event_handlers = {}  # type: Dict[str, Set[Callable]]
         self._images = (
             set()
@@ -150,18 +147,20 @@ class Workbench(tk.Tk):
         self.content_inspector_classes = []  # type: List[Type]
         self._latin_shortcuts = {}  # type: Dict[Tuple[int,int], List[Tuple[Callable, Callable]]]
 
-        self._init_diagnostic_logging()
         self._init_language()
 
         self._active_ui_mode = os.environ.get("THONNY_MODE", self.get_option("general.ui_mode"))
 
         self._init_scaling()
 
-        self._add_main_backends()
-
         self._init_theming()
         self._init_window()
-        self.add_view(ShellView, _("Shell"), "s", visible_by_default=True, default_position_key="A")
+        self.option_add("*Dialog.msg.wrapLength", "8i")
+
+        self.add_view(
+            ShellView, tr("Shell"), "s", visible_by_default=True, default_position_key="A"
+        )
+
         assistance.init()
         self._runner = Runner()
         self._load_plugins()
@@ -176,7 +175,8 @@ class Workbench(tk.Tk):
         assert self._editor_notebook is not None
 
         self._init_program_arguments_frame()
-        self._init_regular_mode_link()
+        # self._init_backend_switcher()
+        self._init_regular_mode_link()  # TODO:
 
         self._show_views()
         # Make sure ShellView is loaded
@@ -199,6 +199,7 @@ class Workbench(tk.Tk):
         self.bind_all("<KeyPress>", self._on_all_key_presses, True)
         self.bind("<FocusOut>", self._on_focus_out, True)
         self.bind("<FocusIn>", self._on_focus_in, True)
+        self.bind("BackendRestart", self._on_backend_restart, True)
 
         self._publish_commands()
         self.initializing = False
@@ -206,7 +207,23 @@ class Workbench(tk.Tk):
         self._make_sanity_checks()
         if self._is_server():
             self._poll_ipc_requests()
+
+        """
+        for name in sorted(sys.modules):
+            if (
+                not name.startswith("_")
+                and not name.startswith("thonny")
+                and not name.startswith("tkinter")
+            ):
+                print(name)
+        """
+
         self.after(1, self._start_runner)  # Show UI already before waiting for the backend to start
+        self.after_idle(self.advertise_ready)
+
+    def advertise_ready(self):
+        self.event_generate("WorkbenchReady")
+        self.ready = True
 
     def _make_sanity_checks(self):
         home_dir = os.path.expanduser("~")
@@ -219,6 +236,7 @@ class Workbench(tk.Tk):
             messagebox.showwarning(
                 "Problems with home directory",
                 bad_home_msg + "\nThis may cause problems for Thonny.",
+                master=self,
             )
 
     def _try_action(self, action: Callable) -> None:
@@ -229,7 +247,7 @@ class Workbench(tk.Tk):
 
     def _init_configuration(self) -> None:
         self._configuration_manager = try_load_configuration(thonny.CONFIGURATION_FILE)
-        self._configuration_pages = []  # type: List[Tuple[str, str, Type[tk.Widget]]
+        self._configuration_pages = []  # type: List[Tuple[str, str, Type[tk.Widget]]]
 
         self.set_default("general.single_instance", thonny.SINGLE_INSTANCE_DEFAULT)
         self.set_default("general.ui_mode", "simple" if running_on_rpi() else "regular")
@@ -238,47 +256,26 @@ class Workbench(tk.Tk):
         self.set_default("general.scaling", "default")
         self.set_default("general.language", languages.BASE_LANGUAGE_CODE)
         self.set_default("general.font_scaling_mode", "default")
+        self.set_default("general.environment", [])
+        self.set_default("file.avoid_zenity", False)
         self.set_default("run.working_directory", os.path.expanduser("~"))
+        self.update_debug_mode()
+
+    def _tweak_environment(self):
+        for entry in self.get_option("general.environment"):
+            if "=" in entry:
+                key, val = entry.split("=", maxsplit=1)
+                os.environ[key] = os.path.expandvars(val)
+            else:
+                logger.warning("No '=' in environment entry '%s'", entry)
+
+    def update_debug_mode(self):
+        os.environ["THONNY_DEBUG"] = str(self.get_option("general.debug_mode", False))
+        thonny.set_logging_level()
 
     def _init_language(self) -> None:
         """Initialize language."""
-        language_code = self.get_option("general.language")
-        if language_code in languages.LANGUAGES_DICT:
-            path = os.path.join(os.path.dirname(__file__), "locale")
-            try:
-                language = gettext.translation("thonny", path, [language_code])
-                language.install()
-            except Exception:
-                traceback.print_exc()
-                self.report_exception("Can't load language " + language_code)
-
-    def _get_logging_level(self) -> int:
-        if self.in_debug_mode():
-            return logging.DEBUG
-        else:
-            return logging.INFO
-
-    def _init_diagnostic_logging(self) -> None:
-        logFormatter = logging.Formatter("%(levelname)s: %(message)s")
-        root_logger = logging.getLogger()
-
-        log_file = os.path.join(THONNY_USER_DIR, "frontend.log")
-        file_handler = logging.FileHandler(log_file, encoding="UTF-8", mode="w")
-        file_handler.setFormatter(logFormatter)
-        file_handler.setLevel(self._get_logging_level())
-        root_logger.addHandler(file_handler)
-
-        console_handler = logging.StreamHandler(sys.stdout)
-        console_handler.setFormatter(logFormatter)
-        console_handler.setLevel(self._get_logging_level())
-        root_logger.addHandler(console_handler)
-
-        root_logger.setLevel(self._get_logging_level())
-
-        import faulthandler
-
-        fault_out = open(os.path.join(THONNY_USER_DIR, "frontend_faults.log"), mode="w")
-        faulthandler.enable(fault_out)
+        languages.set_language(self.get_option("general.language"))
 
     def _init_window(self) -> None:
         self.title("Thonny")
@@ -321,6 +318,9 @@ class Workbench(tk.Tk):
         self.protocol("WM_DELETE_WINDOW", self._on_close)
         self.bind("<Configure>", self._on_configure, True)
 
+    def _init_statusbar(self):
+        self._statusbar = ttk.Frame(self)
+
     def _init_icon(self) -> None:
         # Window icons
         if running_on_linux() and ui_utils.get_tk_version_info() >= (8, 6):
@@ -357,21 +357,18 @@ class Workbench(tk.Tk):
         )  # type: Dict[Tuple[str, str], MenuItem] # key is pair (menu_name, command_label)
 
         # create standard menus in correct order
-        self.get_menu("file", _("File"))
-        self.get_menu("edit", _("Edit"))
-        self.get_menu("view", _("View"))
-        self.get_menu("run", _("Run"))
-        self.get_menu("tempdevice", "Device")
-        self.get_menu("tools", _("Tools"))
-        self.get_menu("help", _("Help"))
+        self.get_menu("file", tr("File"))
+        self.get_menu("edit", tr("Edit"))
+        self.get_menu("view", tr("View"))
+        self.get_menu("run", tr("Run"))
+        self.get_menu("tools", tr("Tools"))
+        self.get_menu("help", tr("Help"))
 
     def _load_plugins(self) -> None:
         # built-in plugins
         import thonny.plugins  # pylint: disable=redefined-outer-name
 
-        self._load_plugins_from_path(
-            thonny.plugins.__path__, "thonny.plugins."  # type: ignore
-        )
+        self._load_plugins_from_path(thonny.plugins.__path__, "thonny.plugins.")  # type: ignore
 
         # 3rd party plugins from namespace package
         try:
@@ -506,37 +503,6 @@ class Workbench(tk.Tk):
 
         self.update_fonts()
 
-    def _add_main_backends(self) -> None:
-        self.set_default("run.backend_name", "SameAsFrontend")
-        self.set_default("CustomInterpreter.used_paths", [])
-        self.set_default("CustomInterpreter.path", "")
-
-        from thonny import running_config_page
-
-        self.add_backend(
-            "SameAsFrontend",
-            running.SameAsFrontendCPythonProxy,
-            _("The same interpreter which runs Thonny (default)"),
-            running_config_page.SameAsFrontEndConfigurationPage,
-            "1",
-        )
-
-        self.add_backend(
-            "CustomCPython",
-            running.CustomCPythonProxy,
-            _("Alternative Python 3 interpreter or virtual environment"),
-            running_config_page.CustomCPythonConfigurationPage,
-            "2",
-        )
-
-        self.add_backend(
-            "PrivateVenv",
-            running.PrivateVenvCPythonProxy,
-            _("A special virtual environment (deprecated)"),
-            running_config_page.PrivateVenvConfigurationPage,
-            "z",
-        )
-
     def _start_runner(self) -> None:
         try:
             self.update_idletasks()  # allow UI to complete
@@ -550,7 +516,9 @@ class Workbench(tk.Tk):
         """Socket will listen requests from newer Thonny instances,
         which try to delegate opening files to older instance"""
 
-        if not self.get_option("general.single_instance") or os.path.exists(thonny.IPC_FILE):
+        if not self.get_option("general.single_instance") or os.path.exists(
+            thonny.get_ipc_file_path()
+        ):
             self._ipc_requests = None
             return
 
@@ -597,16 +565,16 @@ class Workbench(tk.Tk):
 
             secret = str(uuid.uuid4())
 
-            with open(thonny.IPC_FILE, "w") as fp:
+            with open(thonny.get_ipc_file_path(), "w") as fp:
                 fp.write(str(port) + "\n")
                 fp.write(secret + "\n")
 
         else:
             server_socket = socket.socket(socket.AF_UNIX)  # @UndefinedVariable
-            server_socket.bind(thonny.IPC_FILE)
+            server_socket.bind(thonny.get_ipc_file_path())
             secret = ""
 
-        os.chmod(thonny.IPC_FILE, 0o600)
+        os.chmod(thonny.get_ipc_file_path(), 0o600)
         return server_socket, secret
 
     def _init_commands(self) -> None:
@@ -614,7 +582,7 @@ class Workbench(tk.Tk):
         self.add_command(
             "exit",
             "file",
-            _("Exit"),
+            tr("Exit"),
             self._on_close,
             default_sequence=select_sequence("<Alt-F4>", "<Command-q>", "<Control-q>"),
             extra_sequences=["<Alt-F4>"]
@@ -624,14 +592,14 @@ class Workbench(tk.Tk):
             else [],
         )
 
-        self.add_command("show_options", "tools", _("Options..."), self.show_options, group=180)
+        self.add_command("show_options", "tools", tr("Options..."), self.show_options, group=180)
         self.createcommand("::tk::mac::ShowPreferences", self.show_options)
         self.createcommand("::tk::mac::Quit", self._mac_quit)
 
         self.add_command(
             "increase_font_size",
             "view",
-            _("Increase font size"),
+            tr("Increase font size"),
             lambda: self._change_font_size(1),
             default_sequence=select_sequence("<Control-plus>", "<Command-Shift-plus>"),
             extra_sequences=["<Control-KP_Add>"],
@@ -641,7 +609,7 @@ class Workbench(tk.Tk):
         self.add_command(
             "decrease_font_size",
             "view",
-            _("Decrease font size"),
+            tr("Decrease font size"),
             lambda: self._change_font_size(-1),
             default_sequence=select_sequence("<Control-minus>", "<Command-minus>"),
             extra_sequences=["<Control-KP_Subtract>"],
@@ -653,7 +621,7 @@ class Workbench(tk.Tk):
         self.add_command(
             "focus_editor",
             "view",
-            _("Focus editor"),
+            tr("Focus editor"),
             self._cmd_focus_editor,
             default_sequence=select_sequence("<Alt-e>", "<Command-Alt-e>"),
             group=70,
@@ -662,7 +630,7 @@ class Workbench(tk.Tk):
         self.add_command(
             "focus_shell",
             "view",
-            _("Focus shell"),
+            tr("Focus shell"),
             self._cmd_focus_shell,
             default_sequence=select_sequence("<Alt-s>", "<Command-Alt-s>"),
             group=70,
@@ -673,7 +641,7 @@ class Workbench(tk.Tk):
             self.add_command(
                 "toggle_maximize_view",
                 "view",
-                _("Maximize view"),
+                tr("Maximize view"),
                 self._cmd_toggle_maximize_view,
                 flag_name="view.maximize_view",
                 default_sequence=None,
@@ -685,7 +653,7 @@ class Workbench(tk.Tk):
             self.add_command(
                 "toggle_maximize_view",
                 "view",
-                _("Full screen"),
+                tr("Full screen"),
                 self._cmd_toggle_full_screen,
                 flag_name="view.full_screen",
                 default_sequence=select_sequence("<F11>", "<Command-Shift-F>"),
@@ -696,8 +664,8 @@ class Workbench(tk.Tk):
             self.add_command(
                 "font",
                 "tools",
-                _("Change font size"),
-                caption=_("Zoom"),
+                tr("Change font size"),
+                caption=tr("Zoom"),
                 handler=self._toggle_font_size,
                 image="zoom",
                 include_in_toolbar=True,
@@ -706,15 +674,15 @@ class Workbench(tk.Tk):
             self.add_command(
                 "quit",
                 "help",
-                _("Exit Thonny"),
+                tr("Exit Thonny"),
                 self._on_close,
                 image="quit",
-                caption=_("Quit"),
+                caption=tr("Quit"),
                 include_in_toolbar=True,
                 group=101,
             )
 
-        if self.in_debug_mode():
+        if thonny.in_debug_mode():
             self.bind_all("<Control-Shift-Alt-D>", self._print_state_for_debugging, True)
 
     def _print_state_for_debugging(self, event) -> None:
@@ -722,6 +690,7 @@ class Workbench(tk.Tk):
 
     def _init_containers(self) -> None:
 
+        margin = 10
         # Main frame functions as
         # - a backgroud behind padding of main_pw, without this OS X leaves white border
         # - a container to be hidden, when a view is maximized and restored when view is back home
@@ -733,7 +702,7 @@ class Workbench(tk.Tk):
         self._maximized_view = None  # type: Optional[tk.Widget]
 
         self._toolbar = ttk.Frame(main_frame, padding=0)
-        self._toolbar.grid(column=0, row=0, sticky=tk.NSEW, padx=10, pady=(5, 0))
+        self._toolbar.grid(column=0, row=0, sticky=tk.NSEW, padx=margin, pady=(5, 0))
 
         self.set_default("layout.west_pw_width", self.scale(150))
         self.set_default("layout.east_pw_width", self.scale(150))
@@ -746,7 +715,7 @@ class Workbench(tk.Tk):
 
         self._main_pw = AutomaticPanedWindow(main_frame, orient=tk.HORIZONTAL)
 
-        self._main_pw.grid(column=0, row=1, sticky=tk.NSEW, padx=10, pady=10)
+        self._main_pw.grid(column=0, row=1, sticky=tk.NSEW, padx=margin, pady=(margin, 0))
         main_frame.columnconfigure(0, weight=1)
         main_frame.rowconfigure(1, weight=1)
 
@@ -788,9 +757,118 @@ class Workbench(tk.Tk):
             self.set_default("layout.notebook_" + nb_name + "_visible_view", None)
 
         self._editor_notebook = EditorNotebook(self._center_pw)
-        self._editor_notebook.position_key = 1  # type: ignore
+        self._editor_notebook.position_key = 1
         self._center_pw.insert("auto", self._editor_notebook)
 
+        self._statusbar = ttk.Frame(main_frame)
+        self._statusbar.grid(column=0, row=2, sticky="nsew", padx=margin, pady=(0))
+        self._statusbar.columnconfigure(2, weight=2)
+        self._status_label = ttk.Label(self._statusbar, text="")
+        self._status_label.grid(row=1, column=1, sticky="w")
+
+        self._init_backend_switcher()
+
+    def _init_backend_switcher(self):
+
+        # Set up the menu
+        self._backend_conf_variable = tk.StringVar(value="{}")
+        self._backend_menu = tk.Menu(self._statusbar, tearoff=False)
+
+        # Set up the button
+        self._backend_button = ttk.Button(self._statusbar, text="", style="Toolbutton")
+
+        self._backend_button.grid(row=1, column=3, sticky="e")
+        self._backend_button.configure(command=self._post_backend_menu)
+
+    def _post_backend_menu(self):
+        menu_font = tk_font.nametofont("TkMenuFont")
+
+        def choose_backend():
+            backend_conf = ast.literal_eval(self._backend_conf_variable.get())
+            assert isinstance(backend_conf, dict), "backend conf is %r" % backend_conf
+            for name, value in backend_conf.items():
+                self.set_option(name, value)
+            get_runner().restart_backend(False)
+
+        self._backend_menu.delete(0, "end")
+        max_description_width = 0
+        button_text_width = menu_font.measure(self._backend_button.cget("text"))
+
+        num_entries = 0
+        for backend in sorted(self.get_backends().values(), key=lambda x: x.sort_key):
+            entries = backend.proxy_class.get_switcher_entries()
+
+            if not entries:
+                continue
+
+            if len(entries) == 1:
+                self._backend_menu.add_radiobutton(
+                    label=backend.description,
+                    command=choose_backend,
+                    variable=self._backend_conf_variable,
+                    value=repr(entries[0][0]),
+                )
+            else:
+                submenu = tk.Menu(self._backend_menu, tearoff=False)
+                for conf, label in entries:
+                    submenu.add_radiobutton(
+                        label=label,
+                        command=choose_backend,
+                        variable=self._backend_conf_variable,
+                        value=repr(conf),
+                    )
+                self._backend_menu.add_cascade(label=backend.description, menu=submenu)
+
+            max_description_width = max(
+                menu_font.measure(backend.description), max_description_width
+            )
+        num_entries += 1
+
+        # self._backend_conf_variable.set(value=self.get_option("run.backend_name"))
+
+        self._backend_menu.add_separator()
+        self._backend_menu.add_command(
+            label=tr("Configure interpreter..."),
+            command=lambda: self.show_options("interpreter"),
+        )
+
+        post_x = self._backend_button.winfo_rootx()
+        post_y = self._backend_button.winfo_rooty()
+
+        if self.winfo_screenwidth() / self.winfo_screenheight() > 2:
+            # Most likely several monitors.
+            # Tk will adjust x properly with single monitor, but when Thonny is maximized
+            # on a monitor, which has another monitor to its right, the menu can be partially
+            # displayed on another monitor (at least in Ubuntu).
+            width_diff = max_description_width - button_text_width
+            post_x -= width_diff + menu_font.measure("mmm")
+
+        if running_on_mac_os():
+            # won't be good location otherwise
+            popup_entry = num_entries + 4
+        else:
+            popup_entry = ""
+
+        # print(post_x, post_y)
+        try:
+            self._backend_menu.tk_popup(post_x, post_y, entry=popup_entry)
+        except tk.TclError as e:
+            if not 'unknown option "-state"' in str(e):
+                logger.warning("Problem with switcher popup", exc_info=e)
+
+    def _on_backend_restart(self, event):
+        proxy = get_runner().get_backend_proxy()
+        if proxy:
+            desc = proxy.get_clean_description()
+            self._backend_conf_variable.set(value=repr(proxy.get_current_switcher_configuration()))
+        else:
+            backend_conf = self._backends.get(self.get_option("run.backend_name"), None)
+            if backend_conf:
+                desc = backend_conf.description
+            else:
+                desc = "<no backend>"
+        self._backend_button.configure(text=desc)
+
     def _init_theming(self) -> None:
         self._style = ttk.Style()
         self._ui_themes = (
@@ -917,6 +995,10 @@ class Workbench(tk.Tk):
 
             self.event_generate("UICommandDispatched", command_id=command_id, denied=denied)
 
+        def dispatch_if_caps_lock_is_on(event):
+            if caps_lock_is_on(event.state) and not shift_is_pressed(event.state):
+                dispatch(event)
+
         sequence_option_name = "shortcuts." + command_id
         self.set_default(sequence_option_name, default_sequence)
         sequence = self.get_option(sequence_option_name)
@@ -924,6 +1006,18 @@ class Workbench(tk.Tk):
         if sequence:
             if not skip_sequence_binding:
                 self.bind_all(sequence, dispatch, True)
+                # work around caps-lock problem
+                # https://github.com/thonny/thonny/issues/1347
+                # Unfortunately the solution doesn't work with sequences involving Shift
+                # (in Linux with the expected solution Shift sequences did not come through
+                # with Caps Lock, and in Windows, the shift handlers started to react
+                # on non-shift keypresses)
+                # Python 3.7 on Mac seems to require lower letters for shift sequences.
+                parts = sequence.strip("<>").split("-")
+                if len(parts[-1]) == 1 and parts[-1].islower() and "Shift" not in parts:
+                    lock_sequence = "<%s-Lock-%s>" % ("-".join(parts[:-1]), parts[-1].upper())
+                    self.bind_all(lock_sequence, dispatch_if_caps_lock_is_on, True)
+
             # register shortcut even without binding
             register_latin_shortcut(self._latin_shortcuts, sequence, handler, tester)
 
@@ -1091,6 +1185,7 @@ class Workbench(tk.Tk):
 
         # assing names to related classes
         proxy_class.backend_name = name  # type: ignore
+        proxy_class.backend_description = description  # type: ignore
         if not getattr(config_page_constructor, "backend_name", None):
             config_page_constructor.backend_name = name
 
@@ -1102,7 +1197,7 @@ class Workbench(tk.Tk):
         images: Dict[str, str] = {},
     ) -> None:
         if name in self._ui_themes:
-            warn(_("Overwriting theme '%s'") % name)
+            warn(tr("Overwriting theme '%s'") % name)
 
         self._ui_themes[name] = (parent, settings, images)
 
@@ -1110,7 +1205,7 @@ class Workbench(tk.Tk):
         self, name: str, parent: Optional[str], settings: FlexibleSyntaxThemeSettings
     ) -> None:
         if name in self._syntax_themes:
-            warn(_("Overwriting theme '%s'") % name)
+            warn(tr("Overwriting theme '%s'") % name)
 
         self._syntax_themes[name] = (parent, settings)
 
@@ -1269,7 +1364,7 @@ class Workbench(tk.Tk):
         col = 1000
         self._toolbar.columnconfigure(col, weight=1)
 
-        label = ttk.Label(frame, text=_("Program arguments:"))
+        label = ttk.Label(frame, text=tr("Program arguments:"))
         label.grid(row=0, column=0, sticky="nse", padx=5)
 
         self.program_arguments_box = ttk.Combobox(
@@ -1298,7 +1393,7 @@ class Workbench(tk.Tk):
         self.add_command(
             "viewargs",
             "view",
-            _("Program arguments"),
+            tr("Program arguments"),
             toggle,
             flag_name="view.show_program_arguments",
             group=11,
@@ -1312,7 +1407,7 @@ class Workbench(tk.Tk):
 
         label = ttk.Label(
             self._toolbar,
-            text=_("Switch to\nregular\nmode"),
+            text=tr("Switch to\nregular\nmode"),
             justify="right",
             font="SmallLinkFont",
             style="Url.TLabel",
@@ -1323,16 +1418,26 @@ class Workbench(tk.Tk):
         def on_click(event):
             self.set_option("general.ui_mode", "regular")
             tk.messagebox.showinfo(
-                _("Regular mode"),
-                _(
+                tr("Regular mode"),
+                tr(
                     "Configuration has been updated. "
                     + "Restart Thonny to start working in regular mode.\n\n"
                     + "(See 'Tools → Options → General' if you change your mind later.)"
                 ),
+                master=self,
             )
 
         label.bind("<1>", on_click, True)
 
+    def _switch_backend_group(self, group):
+        pass
+
+    def _switch_darkness(self, mode):
+        pass
+
+    def _switch_to_regular_mode(self):
+        pass
+
     def log_program_arguments_string(self, arg_str: str) -> None:
         arg_str = arg_str.strip()
         self.set_option("run.program_arguments", arg_str)
@@ -1409,6 +1514,11 @@ class Workbench(tk.Tk):
             name: meant to be used as not translatable menu name
             label: translated label, used only when menu with given name doesn't exist yet
         """
+
+        # For compatibility with plug-ins
+        if name in ["device", "tempdevice"] and label is None:
+            label = tr("Device")
+
         if name not in self._menus:
             if running_on_mac_os():
                 conf = {}
@@ -1520,7 +1630,7 @@ class Workbench(tk.Tk):
 
         Args:
             view_id: View class name
-            without package name (eg. 'ShellView') """
+            without package name (eg. 'ShellView')"""
 
         if view_id == "MainFileBrowser":
             # Was renamed in 3.1.1
@@ -1536,9 +1646,7 @@ class Workbench(tk.Tk):
 
         if view.hidden:  # type: ignore
             notebook.insert(
-                "auto",
-                view.home_widget,  # type: ignore
-                text=self._view_records[view_id]["label"],
+                "auto", view.home_widget, text=self._view_records[view_id]["label"]  # type: ignore
             )
             view.hidden = False  # type: ignore
             if hasattr(view, "on_show"):  # type: ignore
@@ -1562,7 +1670,7 @@ class Workbench(tk.Tk):
             # TODO: handle the case, when view is maximized
             view = self._view_records[view_id]["instance"]
             if view.hidden:
-                return
+                return True
 
             if hasattr(view, "before_hide") and view.before_hide() == False:
                 return False
@@ -1573,7 +1681,7 @@ class Workbench(tk.Tk):
             self.event_generate("HideView", view=view, view_id=view_id)
             view.hidden = True
 
-        return None
+        return True
 
     def event_generate(self, sequence: str, event: Optional[Record] = None, **kwargs) -> None:
         """Uses custom event handling when sequence doesn't start with <.
@@ -1623,15 +1731,14 @@ class Workbench(tk.Tk):
             self._event_handlers[sequence].add(func)
 
     def unbind(self, sequence: str, func=None) -> None:
+        # pylint: disable=arguments-differ
         if sequence.startswith("<"):
             tk.Tk.unbind(self, sequence, funcid=func)
         else:
             try:
                 self._event_handlers[sequence].remove(func)
             except Exception:
-                logging.getLogger("thonny").exception(
-                    "Can't remove binding for '%s' and '%s'", sequence, func
-                )
+                logger.exception("Can't remove binding for '%s' and '%s'", sequence, func)
 
     def in_heap_mode(self) -> bool:
         # TODO: add a separate command for enabling the heap mode
@@ -1642,13 +1749,17 @@ class Workbench(tk.Tk):
         )
 
     def in_debug_mode(self) -> bool:
-        return os.environ.get("THONNY_DEBUG", False) in [
-            "1",
-            1,
-            "True",
-            True,
-            "true",
-        ] or self.get_option("general.debug_mode", False)
+        return (
+            os.environ.get("THONNY_DEBUG", False)
+            in [
+                "1",
+                1,
+                "True",
+                True,
+                "true",
+            ]
+            or self.get_option("general.debug_mode", False)
+        )
 
     def _init_scaling(self) -> None:
         self._default_scaling_factor = self.tk.call("tk", "scaling")
@@ -1831,7 +1942,6 @@ class Workbench(tk.Tk):
 
         button = ttk.Button(
             group_frame,
-            command=handler,
             image=image_spec,
             style="Toolbutton",
             state=tk.NORMAL,
@@ -1840,6 +1950,17 @@ class Workbench(tk.Tk):
             pad=(10, 0) if self.in_simple_mode() else None,
             width=button_width,
         )
+
+        def toolbar_handler(*args):
+            handler(*args)
+            self._update_toolbar()
+            if self.focus_get() == button:
+                # previously selected widget would be better candidate, but this is
+                # better than button
+                self._editor_notebook.focus_set()
+
+        button.configure(command=toolbar_handler)
+
         button.pack(side=tk.LEFT)
         button.tester = tester  # type: ignore
         tooltip_text = command_label
@@ -1958,9 +2079,7 @@ class Workbench(tk.Tk):
         while widget is not None:
             if hasattr(widget, "home_widget"):
                 # if widget is view, then widget.master is workbench
-                widget.grid(
-                    row=1, column=0, sticky=tk.NSEW, in_=widget.master  # type: ignore
-                )
+                widget.grid(row=1, column=0, sticky=tk.NSEW, in_=widget.master)  # type: ignore
                 # hide main_frame
                 self._main_frame.grid_forget()
                 self._maximized_view = widget
@@ -1989,6 +2108,9 @@ class Workbench(tk.Tk):
 
         ui_utils.show_dialog(dlg)
 
+        if dlg.backend_restart_required:
+            get_runner().restart_backend(False)
+
     def _cmd_focus_editor(self) -> None:
         self.get_editor_notebook().focus_set()
 
@@ -2121,7 +2243,7 @@ class Workbench(tk.Tk):
             self.after(50, self._poll_ipc_requests)
 
     def _on_close(self) -> None:
-        if not self.get_editor_notebook().check_allow_closing():
+        if self._editor_notebook and not self._editor_notebook.check_allow_closing():
             return
 
         self._closing = True
@@ -2130,6 +2252,12 @@ class Workbench(tk.Tk):
             self._editor_notebook.remember_open_files()
             self.event_generate("WorkbenchClose")
             self._configuration_manager.save()
+            temp_dir = self.get_temp_dir(create_if_doesnt_exist=False)
+            if os.path.exists(temp_dir):
+                try:
+                    shutil.rmtree(temp_dir)
+                except Exception as e:
+                    logger.error("Could not remove temp dir", exc_info=e)
         except Exception:
             self.report_exception()
 
@@ -2160,8 +2288,8 @@ class Workbench(tk.Tk):
 
     def destroy(self) -> None:
         try:
-            if self._is_server() and os.path.exists(thonny.IPC_FILE):
-                os.remove(thonny.IPC_FILE)
+            if self._is_server() and os.path.exists(thonny.get_ipc_file_path()):
+                os.remove(thonny.get_ipc_file_path())
 
             self._closing = True
 
@@ -2234,7 +2362,9 @@ class Workbench(tk.Tk):
                     # was renamed in 2.2b5
                     view_name = "VariablesView"
 
-                if self.get_ui_mode() != "simple" or view_name in SIMPLE_MODE_VIEWS:
+                if (
+                    self.get_ui_mode() != "simple" or view_name in SIMPLE_MODE_VIEWS
+                ) and view_name in self._view_records:
                     self.show_view(view_name)
 
         # make sure VariablesView is at least loaded
@@ -2343,6 +2473,8 @@ class Workbench(tk.Tk):
             return
 
         # Fallback
+        import webbrowser
+
         webbrowser.open(url, False, True)
 
     def open_help_topic(self, topic, fragment=None):
@@ -2361,6 +2493,12 @@ class Workbench(tk.Tk):
     def get_toolbar(self):
         return self._toolbar
 
+    def get_temp_dir(self, create_if_doesnt_exist=True):
+        path = os.path.join(THONNY_USER_DIR, "temp")
+        if create_if_doesnt_exist:
+            os.makedirs(path, exist_ok=True)
+        return path
+
 
 class WorkbenchEvent(Record):
     def __init__(self, sequence: str, **kwargs) -> None:
